/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.common;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.UUID;

import org.mindswap.pellet.jena.PelletReasonerFactory;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

import com.hp.hpl.jena.ontology.OntDocumentManager;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.reasoner.ValidityReport;
import com.hp.hpl.jena.reasoner.ValidityReport.Report;

/**
 * JenaStarter.java
 * 
 * Create and updates the Document manager of the ontology parser Jena. WARNING :
 * the owl file must contains the "xml:base" property
 * 
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">Sebastien Bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * @author <a href="mailto:Graham.Thomson@inria.fr">Graham Thomson</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 */
public class JenaStarter {

	/**
	 * The property manager used by this class
	 */
	private static ConfigManager configManager = ConfigManager.getInstance();

	/**
	 * The instance of Jena Document Manager that is up to date.
	 */
	private static OntDocumentManager manager = OntDocumentManager.getInstance();

	/**
	 * Contains the association of the distance URI of a file and the URI of the
	 * local copy of this file.
	 */
	private static TreeMap<String, String> mapOfURI = new TreeMap<String, String>();

	// donne before any call of method/constructor in this class
	static {
		loadMapping();
		// DebugPrinter.print(mapOfURI,"mapOfURI",DebugPrinter.INFO);
	}

	/**
	 * instance of the Global ontology.
	 */
	// private static OntModel globalOntology = null;
	/**
	 * Update the document manager by filling the map between distant and local
	 * files
	 */
	private static void loadMapping() {

		File f = new File(configManager.getOntologyFolder());
		loadMapping(f);
		// System.out.println(">>>JenaStarter::loadMapping() file="+f);
		// System.out.println(">>>JenaStarter::loadMapping()
		// fileURI="+f.toURI().toString());

	}

	/**
	 * Update the document manager by filling the map between distant and local
	 * files
	 * 
	 * @param f :
	 *            the current File (which permit recursion if the file is a
	 *            directory
	 */
	private static void loadMapping(File f) {
		if (f.isDirectory() && !f.isHidden() && !f.getName().equals(".svn")) {
			File[] listOfFiles = f.listFiles();
			for (int i = 0; i < listOfFiles.length; i++) {
				loadMapping(listOfFiles[i]);
			}
		} else {
			if (f.isFile()) {
				try {
					JenaStarter.updateManager(Tools.getURIStringFromFilePath(f.getAbsolutePath()));
					// DebugPrinter.print("Loading onto: " +
					// f.getAbsolutePath(), 1);
				} catch (IOException e) {
					DebugPrinter.print("An error occured during read of file "
							+ f.getAbsolutePath() + ". (More: "
							+ e.getMessage() + ")", 2);
				} catch (SAXException e) {
					DebugPrinter.print("An error occured during parse of file "
							+ f.getAbsolutePath() + ". (More: "
							+ e.getMessage() + ")", 2);
				}
			}
		}

	}

	/**
	 * This method add an entry into the map of the ontology Document Manager.
	 * It also add an entry into the local map "mapOfURI". The value is caught
	 * into a file thanks to the "xml:base" property
	 * 
	 * @param fileName :
	 *            the uri of the xml file
	 * @throws IOException :
	 *             if an error occurs during the read of file
	 * @throws SAXException :
	 *             if an error occurs during the parsing
	 */
	private static void updateManager(final String fileName)
			throws IOException, SAXException {

		XMLReader myReader = XMLReaderFactory.createXMLReader();

		// creation of the SAX handler
		DefaultHandler handler = new DefaultHandler() {

			@Override
			public void startElement(String uri, String localName,
					String qName, Attributes attributes) throws SAXException {

				// get the "xml:base" attribute of the RDF tag
				if (localName.equals("RDF")) {
					try {
						String value = attributes.getValue(0);

						// if the string finish with a '#', remove it !
						if (value.endsWith("#")) {
							value = value.substring(0, value.length() - 1);
						}

						mapOfURI.put(value, fileName);
						manager.addAltEntry(value, fileName);
					} catch (Exception e) {
						throw new SAXException(
								"The RDF attribute \"xml:base\" doesn't contain any value");
					}
				}

			}

		};

		// put the handler in the sax machine
		myReader.setContentHandler(handler);
		myReader.setErrorHandler(handler);

		// parse the file
		// TODO myReader.parse(new InputSource(new FileReader(new
		// File(URI.create(JenaStarter.correctURI(fileName))))));
		myReader.parse(new InputSource(new FileReader(new File(
				URI.create(fileName)))));
	}

	/**
	 * Create an instance of OntModel, using the updated Document Manager<br>
	 * <b>WARNING :</b> do not use the reasoner for parsing ! If you do, it
	 * will result to a "Memory overhead" problem
	 * 
	 * @param useReasoner
	 *            indicate if the reasoner has to be used. If <i>true</i> the
	 *            reasoner included in Jena will be used, otherwise it will not be used !
	 * @return an OntModel Object
	 */
	public static OntModel getInstanceOfOntModel(boolean useReasoner) {
		if (useReasoner) {
			// Use Jena's OWL DL reasoner.
			// OntModelSpec s = new
			// OntModelSpec(OntModelSpec.OWL_DL_MEM_RULE_INF);
			OntModelSpec s = new OntModelSpec(PelletReasonerFactory.THE_SPEC);	
			return ModelFactory.createOntologyModel(s);
		} else
			return ModelFactory.createOntologyModel();
	}

	/**
	 * Create a File in the LOCALDIRCTORY with the contains give in parameter.
	 * WARNING : the content MUST BE a correct ontology File with a "xml:base"
	 * Attribute associated to RDF tag
	 * 
	 * @param fileContent :
	 *            the description od the ontology
	 * @throws IOException :
	 *             if an error occurs during parsing of the file
	 * @throws URISyntaxException
	 *             if an error occurs during the creation of the URI
	 */
	public static void addOntologyFile(String fileContent) throws IOException,
			URISyntaxException {

		File destinationFile = null;
		String fileName = "";
		URI _URI;

		// creation of the File
		do {

			// generate a unique name for the file
			fileName = UUID.randomUUID().toString() + ".owl_add";
			// fileName = fileContent.hashCode() + ".owl_add";

			_URI = Tools.getURIFromFilePath(configManager.getOntologyFolder()
					+ configManager.FILE_SEPARATOR + fileName);
			// try {

			destinationFile = new File(_URI);
			/*
			 * } catch (URISyntaxException e) { System.err .println("An error
			 * occurs during creation of the file " + fileName + ". (More:" +
			 * e.getMessage() + ")"); destinationFile = new
			 * File(JenaStarter.LOCALDIRECTORY + JenaStarter.FILE_SEPARATOR +
			 * fileName); }
			 */
		} while (destinationFile.exists());

		// really create the file
		destinationFile.createNewFile();
		// create the stream
		BufferedWriter writer = new BufferedWriter(new FileWriter(
				destinationFile));
		// write the content of the file
		writer.write(fileContent);
		// flush the buffer
		writer.flush();
		// close the stream
		writer.close();

		// update the ontology File Manager
		try {
			JenaStarter.updateManager(_URI.toString());
		} catch (SAXException e) {
			DebugPrinter.print("An error occurred during creation of the file "
					+ fileName + ". (More:" + e.getMessage() + ")", 2);
			destinationFile.delete();
		}

	}

	// public static void main(String args[]) {
	// try {
	// JenaStarter.getInstanceOfOntModel();
	// System.out.println(JenaStarter.mapOfURI);
	//
	// String fileContent = "";
	//
	// BufferedReader in = new BufferedReader(new FileReader("test.txt"));
	// String read = "";
	// while ((read = in.readLine()) != null)
	// fileContent += read + "\n";
	//
	// JenaStarter.addOntologyFile(fileContent);
	// } catch (IOException e) {
	// e.printStackTrace();
	// } catch (URISyntaxException e) {
	// e.printStackTrace();
	// }
	//
	// System.out.println(JenaStarter.mapOfURI);
	// }

	/**
	 * Get the instance of the main Ontology
	 * 
	 * @return the globalOntology
	 */
	/*
	 * public static OntModel getGlobalOntology() { if (globalOntology == null)
	 * globalOntology = getInstanceOfOntModel(); return globalOntology; }
	 */

	/**
	 * @param globalOntology
	 *            the globalOntology to set
	 */
	/*
	 * public static void setGlobalOntology(OntModel globalOntology) {
	 * JenaStarter.globalOntology = globalOntology; }
	 */

	public static void main(String[] args) {
		OntModel model = JenaStarter.getInstanceOfOntModel(false);
		model.read(Tools.getURIStringFromFilePath("C:\\Documents and Settings\\Graham\\workspace\\COCOA\\owl_files\\MultimediaBrowserQoS.owl"));
		ValidityReport reports = model.validate();
		Iterator itr = reports.getReports();
		while (itr.hasNext()) {
			Report report = (Report) itr.next();
			DebugPrinter.print(report.toString(), 1);
		}

	}

	/**
	 * Return the URI of the file that contains the ontology with such
	 * ontologyURI
	 * 
	 * @param ontologyURI :
	 *            the URI of the ontology requested
	 * @return the name of the file of the requested ontology
	 */
	public static String getOntologyDescriptionFile(String ontologyURI) {
		return mapOfURI.get(ontologyURI);
	}
}
