package fr.inria.icocoa.servicemodel.matching;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.exception.SemanticTypeNotKnowException;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.IO;
import fr.inria.icocoa.servicemodel.Input;
import fr.inria.icocoa.servicemodel.Output;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.UserTask;
import fr.inria.icocoa.servicemodel.semantictype.SemanticType;

/**
 * This class implements a Matcher that use synthaxic then semantic (equivalent
 * concept) to match components (allows less inputs than describe in the user
 * task, and more output). <br>
 * The ontology that will be used is load thanks the first semantic type found
 * during the matching process, and this ontology will be the one that will be
 * used during the all matching process. To change it, you must create a new
 * Matcher.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Sep 06, 2007 3:40:39 PM
 */
public class ExactSemanticMatcher extends SemanticMatcher {

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.matching.Matcher#capabilityMatch(fr.inria.icocoa.servicemodel.Capability,
	 *      fr.inria.icocoa.servicemodel.Capability)
	 */
	@Override
	public CapabilityMatch capabilityMatch(Capability serviceCapa,
			Capability taskCapa) throws SemanticTypeNotKnowException {
		// increase the counter of Capabilities matched
		stats[1]++;

		String semanticType = taskCapa.getType();

		if (this.isSemanticlyEquivalent(semanticType, serviceCapa.getType())) {
			// generate result
			return this.generateMatching(serviceCapa, taskCapa);
		} else
			return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.matching.Matcher#ioMatch(java.lang.String,
	 *      java.lang.String)
	 */
	@Override
	public int ioMatch(String serviceIOSemanticType, String taskIOsemanticType)
			throws SemanticTypeNotKnowException {
		// increase the counter of IO matched
		stats[2]++;
		return this.typeMatch(serviceIOSemanticType, taskIOsemanticType);		
	}
	

	
	

	/* (non-Javadoc)
	 * @see fr.inria.icocoa.servicemodel.matching.Matcher#typeMatch(fr.inria.icocoa.servicemodel.semantictype.SemanticType, fr.inria.icocoa.servicemodel.semantictype.SemanticType)
	 */
	@Override
	public short typeMatch(SemanticType serviceType, SemanticType taskType) throws SemanticTypeNotKnowException {
		return this.typeMatch(serviceType.getType(), taskType.getType());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.matching.Matcher#serviceMatch(fr.inria.icocoa.servicemodel.Service,
	 *      fr.inria.icocoa.servicemodel.UserTask)
	 */
	@Override
	public ServiceMatch serviceMatch(Service service, UserTask userTask)
			throws SemanticTypeNotKnowException {
		// increase the counter of Services matched
		stats[0]++;

		ServiceMatch result = this.partialExactMatch(service, userTask);

		// if no matching
		if (result == null)
			return null;

		// test the number of Functionality matching (all required need to me
		// matched)
		if (result.getCapabilityMatches().size() < (userTask.getRequired_Capabilities().size() + userTask.getCapabilitiesMap().size()))
			return null;

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.matching.Matcher#ioMatch(fr.inria.icocoa.servicemodel.IO,
	 *      fr.inria.icocoa.servicemodel.IO)
	 */
	@Override
	public IOMatch ioMatch(IO serviceIO, IO taskIO)
			throws SemanticTypeNotKnowException {
		if (serviceIO.getClass().equals(taskIO.getClass())) {
			if (this.ioMatch(serviceIO.getType(), taskIO.getType()) == MatchingType.EXACT) {
				return new IOMatch(taskIO, serviceIO, MatchingType.EXACT);
			}
		}
		return null;
	}

	/**
	 * Generate the complete FunctionalityMatching object corresponding to the
	 * matching between this Capability and the one give as parameter
	 * 
	 * @param serviceCapa
	 *            the current Capability
	 * @param taskCapa
	 *            the capability to match with.
	 * @return a MatchingCapabibility Object containing all information, if the
	 *         the matching is possible, <i>null</i> otherwise.
	 * @throws SemanticTypeNotKnowException
	 *             if one type is not found
	 */
	private CapabilityMatch generateMatching(Capability serviceCapa,
			Capability taskCapa) throws SemanticTypeNotKnowException {

		// generate IOMatching
		Map<IO, IO> iomatching = this.ioExactMatch(serviceCapa, taskCapa);

		// if no io matching
		if (iomatching == null)
			return null;

		// create the matching at Capability level
		CapabilityMatch result = new CapabilityMatch(taskCapa, serviceCapa,
				MatchingType.EXACT);

		// add matching at io level
		for (Iterator<Entry<IO, IO>> it = iomatching.entrySet().iterator(); it.hasNext();) {
			Entry<IO, IO> entry = it.next();
			IO serviceIO = entry.getKey();
			IO taskIO = entry.getValue();
			if (serviceIO instanceof fr.inria.icocoa.servicemodel.Input)
				result.addIOMatching(new IOMatch(taskIO, serviceIO,
						MatchingType.EXACT));
			if (serviceIO instanceof fr.inria.icocoa.servicemodel.Output)
				result.addIOMatching(new IOMatch(taskIO, serviceIO,
						MatchingType.EXACT));
		}
		return result;
	}

	/**
	 * Try to match the IO of the given capabilities with this (used as UserTask
	 * capability)
	 * 
	 * @param serviceCapa
	 *            the current Capability
	 * @param taskCapa
	 *            the capability to match with.
	 * @return the Map of matching typeName, null if we can't match (not the
	 *         exactly the same type or not the same number)
	 * @throws SemanticTypeNotKnowException
	 *             if one type is not found
	 */
	private Map<IO, IO> ioExactMatch(Capability serviceCapa, Capability taskCapa)
			throws SemanticTypeNotKnowException {
		Map<IO, IO> matchs = new HashMap<IO, IO>();

		/*
		 * Match Inputs (the capa (service capa) could have less inputs the
		 * Usertask)
		 */
		if (taskCapa.getInputs().size() <= serviceCapa.getInputs().size()) {

			/* For each input of selected service capability... */
			for (Iterator<Input> it = taskCapa.getInputs().values().iterator(); it.hasNext();) {
				Input toFind = it.next();

				String type = toFind.getType();
				boolean found = false;

				/*
				 * ...Look if there is an corresponding input for task
				 * capability
				 */
				loop1: for (Iterator<Input> it2 = serviceCapa.getInputs().values().iterator(); !found
						&& it2.hasNext();) {
					Input current = it2.next();

					// test matching between task IO "toFind" and service IO
					// "current"
					if (current.match(type, this) == MatchingType.EXACT) {
						matchs.put(current, toFind);
						// stop the loop to speed up the process
						found = true;
						break loop1;
					}
				}
				if (!found) {
					return null;
				}
			}
		} else {
			return null;
		}

		/*
		 * Match output (the capa (service capa) could have more outputs the
		 * Usertask)
		 */
		if (taskCapa.getOutputs().size() >= serviceCapa.getOutputs().size()) {

			/* For each output of usertask capability... */
			for (Iterator<Output> it = serviceCapa.getOutputs().values().iterator(); it.hasNext();) {
				Output toFind = it.next();

				String type = toFind.getType();
				boolean found = false;
				/*
				 * ...Look if there is an corresponding input for selected
				 * service capability
				 */
				for (Iterator<Output> it2 = taskCapa.getOutputs().values().iterator(); it2.hasNext()
						&& !found;) {
					Output current = it2.next();
					if (current.match(type, this) == MatchingType.EXACT) {
						matchs.put(toFind, current);
						// stop the loop to speed up the process
						found = true;

					}
				}
				if (!found) {
					return null;
				}
			}
		} else {
			return null;
		}
		return matchs;
	}

	/**
	 * Get a partial matching between the task and the current service
	 * 
	 * @param service
	 *            the current service
	 * @param userTask
	 *            the task to match with
	 * @return a ServiceMatch object containing the matching information; if
	 *         some capabilities could be matched, <i>null</i> if no matching
	 *         is possible
	 * @throws SemanticTypeNotKnowException
	 *             if a type is not knowed
	 */
	private ServiceMatch partialExactMatch(Service service, UserTask userTask)
			throws SemanticTypeNotKnowException {

		ServiceMatch result = new ServiceMatch(userTask, service,
				MatchingType.EXACT);

		/* Get the required capabilities */
		List<Capability> required = userTask.getRequiredCapabilities();

		// check all required capability
		for (Iterator<Capability> it = required.iterator(); it.hasNext();) {
			Capability required_capa = it.next();

			List<String> equivalents = this.listEquivalentConcept(required_capa.getType());

			for (String equivalent : equivalents) {
				// match with all capability with the same/equivalent type
				List<Capability> sameType = service.getCapabilityFromTypeName(equivalent);

				// try to match at CapabilityLevel (<=> test io)
				for (Iterator<Capability> it2 = sameType.iterator(); it2.hasNext();) {
					Capability service_capa = it2.next();
					CapabilityMatch cm = service_capa.match(required_capa, this);
					if (cm != null)
						result.addCapabilityMatch(cm);
				}
			}
		}

		/* Get the other capabilities */
		List<Capability> subcapabilties = userTask.getCapabilities();

		// check other capability
		for (Iterator<Capability> it = subcapabilties.iterator(); it.hasNext();) {
			Capability subcapa = it.next();

			List<String> equivalents = this.listEquivalentConcept(subcapa.getType());

			for (String equivalent : equivalents) {
				// match with all capability with the same/equivalent type
				List<Capability> sameType = service.getCapabilityFromTypeName(equivalent);

				// try to match at CapabilityLevel (<=> test io)
				for (Iterator<Capability> it2 = sameType.iterator(); it2.hasNext();) {
					Capability service_capa = it2.next();
					CapabilityMatch cm = service_capa.match(subcapa, this);
					if (cm != null)
						result.addCapabilityMatch(cm);
				}
			}
		}

		// if no matching
		if (result.getCapabilityMatches().size() == 0)
			return null;

		return result;
	}
	
	/**
	 * Get a matching between the two concepts (the task Concept comparing to
	 * the service one)
	 * 
	 * @param serviceTypeName
	 *            the name of the reference concept
	 * @param taskTypeName
	 *            the name of the concept to compare with
	 * @return
	 *            <ul>
	 *            <li>MatchingType.EXACT if the two concept are equals or
	 *            equivalent</li>
	 *            <li>Matching.SUBSUMES if task concept subsums the service one
	 *            </li>
	 *            <li>Matching.PLUGIN if service concept subsums the task one
	 *            </li>
	 *            <li>Matching.FAIL if there is no link between the two concept</li>
	 *            </ul>
	 * @throws SemanticTypeNotKnowException 
	 */
	private short typeMatch(String serviceTypeName, String taskTypeName) throws SemanticTypeNotKnowException {
		if (this.isSemanticlyEquivalent(serviceTypeName,
				taskTypeName))
				return MatchingType.EXACT;
			else
				return MatchingType.FAIL;
	}

}
