/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.parsers.jenaparser;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.PropertyManager;
import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.servicemodel.GroundingProperty;
import fr.inria.icocoa.servicemodel.parsers.jenaparser.tools.ExtendedTools;

/**
 * GroundingParser.java<br>
 * This parser permit to get the Grounding information
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Jan 30, 2007 1:59:37 PM
 */
public class GroundingParser {

	/**
	 * The property manager used by this class
	 */
	private PropertyManager propertyManager = PropertyManager.getInstance();

	/**
	 * the current Ontology model
	 */
	private OntModel model;

	/**
	 * the map that will contain association ConversationURI ->
	 * GroundingProperties
	 */
	private Map<String, GroundingProperty> goundings = new HashMap<String, GroundingProperty>();

	/**
	 * this map will contain the association IO_semanticName -> IO_realWSDLName
	 */
	private Map<String, String> io_mapping = new HashMap<String, String>();

	/**
	 * Constructor of the Object
	 * 
	 * @param model:
	 *            the current ontology model used during parsing
	 */
	public GroundingParser(OntModel model) {
		this.model = model;
	}

	/**
	 * This method parse the process and fill the map that associate an atomic
	 * conversation to an atomic grounding
	 * 
	 * @param file_uri :
	 *            the name of the file to parse to fill the map.
	 * @throws ParsingException
	 *             if an error occurs during parsing
	 */
	public void parseService() throws ParsingException {
		// get list of grounding
		List<Individual> lgrounding = this.getAllAtomicGrounding();

		// for each grounding, get all grounding property and store it in the
		// map
		for (Iterator<Individual> it = lgrounding.iterator(); it.hasNext();) {
			GroundingProperty grounding = this.parseGrounding(it.next());
			this.goundings.put(grounding.getConversationURI(), grounding);
		}
	}

	/**
	 * This method parse the file to get all information associated to the given
	 * Indidual
	 * 
	 * @param iGrounding :
	 *            the individual that is a "WsdlAtomicProcessGrouding
	 * @return an object GroundingProperty that contains all information
	 *         concerning the grounding of an operation
	 * @throws ParsingException
	 *             if an error occurs during parsing
	 */
	private GroundingProperty parseGrounding(Individual iGrounding)
			throws ParsingException {
		try {
			GroundingProperty gp = new GroundingProperty();

			// get the name of WSDd File
			gp.setWsdlFileName(this.getWsdlDocumentName(iGrounding));

			// get the process which the grounding is associated
			gp.setConversationURI(this.getOWLSProcess(iGrounding));

			// get message input name
			gp.setWsdlInputMessage(this.getWSDLInputMessage(iGrounding));

			// get message output name
			gp.setWsdlOutputMessage(this.getWSDLOutputMessage(iGrounding));

			// fill the operation name and porttype
			this.getWSDLOperation(iGrounding, gp);

			// gather all inputs
			this.gatherInputMapping(iGrounding);

			// gather all outputs
			this.gatherOutputMapping(iGrounding);

			return gp;
		} catch (NoSuchElementException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
			throw new ParsingException("The Individual named "
					+ iGrounding.getLocalName() + " is not recognize. (More : "
					+ e.getMessage() + ")");
		}
	}

	/**
	 * This method list all Indivual that are instance of AtomicProcessGrounding
	 * 
	 * @return a list of AtomicProcessGrounding
	 */
	@SuppressWarnings("unchecked")
	private List<Individual> getAllAtomicGrounding() {
		Resource groundingRes = model.getOntResource(this.propertyManager.__OWLS_WSDL_ATOMIC_PROCESS_GROUNDING);
		try {
			List<Individual> lgrounding = model.listIndividuals(groundingRes).toList();
			return lgrounding;
		} catch (NullPointerException e) {
			// if there is no Atomic Process
			return new ArrayList<Individual>();
		}
	}

	/**
	 * This method parse the file to get the name of the WSDL file
	 * 
	 * @param iGrounding :
	 *            the individual that is a "WsdlAtomicProcessGrouding"
	 * @return the name of the WSDL corresponding to the capability, null if
	 *         none is associated
	 * @throws ParsingException
	 *             if an error occurs during parsing
	 */
	@SuppressWarnings("unchecked")
	private String getWsdlDocumentName(Individual iGrounding)
			throws ParsingException {
		Property type = model.createProperty(this.propertyManager.__OWLS_WSDL_DOCUMENT);
		List<Statement> lWsdlDocument = iGrounding.listProperties(type).toList();
		try {
			return ExtendedTools.getURIFromStatement(lWsdlDocument.iterator().next());
		} catch (NullPointerException e) {
			return null;
		}
	}

	/**
	 * This method parse the file to get the name of the conversation associated
	 * to the Grounding
	 * 
	 * @param iGrounding :
	 *            the individual that is a "WsdlAtomicProcessGrouding"
	 * @return the name of conversation corresponding to the capability, null if
	 *         none is associated
	 * @throws ParsingException
	 *             if an error occurs during parsing
	 */
	@SuppressWarnings("unchecked")
	private String getOWLSProcess(Individual iGrounding)
			throws ParsingException {
		Property type = model.createProperty(this.propertyManager.__OWLS_GROUNDING_PROCESS);
		List<Statement> lOwlsProcess = iGrounding.listProperties(type).toList();
		try {
			return ExtendedTools.getURIFromStatement(lOwlsProcess.iterator().next());
		} catch (NullPointerException e) {
			return null;
		}
	}

	/**
	 * This method parse the file to get the name of the messages send to the
	 * Web Service associated to the Grounding
	 * 
	 * @param iGrounding :
	 *            the individual that is a "WsdlAtomicProcessGrouding"
	 * @return the name of the input message
	 * @throws ParsingException
	 *             if an error occurs during parsing
	 */
	@SuppressWarnings("unchecked")
	private String getWSDLInputMessage(Individual iGrounding)
			throws ParsingException {
		Property type = model.createProperty(this.propertyManager.__OWLS_WSDL_INPUTMESSAGE);
		List<Statement> lInputMessages = iGrounding.listProperties(type).toList();
		try {
			return ExtendedTools.getURIFromStatement(lInputMessages.iterator().next());
		} catch (NullPointerException e) {
			return null;
		}
	}

	/**
	 * This method parse the file to get the name of the messages receive from
	 * the Web Service associated to the Grounding
	 * 
	 * @param iGrounding :
	 *            the individual that is a "WsdlAtomicProcessGrouding"
	 * @return the name of the output message
	 * @throws ParsingException
	 *             if an error occurs during parsing
	 */
	@SuppressWarnings("unchecked")
	private String getWSDLOutputMessage(Individual iGrounding)
			throws ParsingException {
		Property type = model.createProperty(this.propertyManager.__OWLS_WSDL_OUTPUTMESSAGE);
		List<Statement> lInputMessages = iGrounding.listProperties(type).toList();
		try {
			return ExtendedTools.getURIFromStatement(lInputMessages.iterator().next());
		} catch (NullPointerException e) {
			return null;
		}
	}

	/**
	 * This method will add the value of field OperationName and OperationType
	 * to the object GroundingProperty
	 * 
	 * @param iGrounding
	 *            the individual node, from which the method will gather
	 *            information
	 * @param gp :
	 *            the object to fill
	 * @throws ParsingException
	 *             if an error occurs during parsing
	 */
	@SuppressWarnings("unchecked")
	private void getWSDLOperation(Individual iGrounding, GroundingProperty gp)
			throws ParsingException {

		// get List of WSDLOperation
		Property wotype = model.createProperty(this.propertyManager.__OWLS_WSDL_OPERATION);
		List<Statement> wsdlOperations = iGrounding.listProperties(wotype).toList();

		try {
			if (wsdlOperations.size() > 0) {
				// get Individual Corresponding to wsdlOperation
				Statement state = wsdlOperations.iterator().next();

				// get Resource object correspondingh to this Statement (no name
				// define, so, we can't access to the individual directly)
				Resource wsdlOperationRefs = state.getResource();

				if (wsdlOperationRefs.listProperties().toList().size() > 0) {

					// get the OperationName property
					Property opeNameType = model.createProperty(this.propertyManager.__OWLS_GROUNDING_OPERATION);
					List<Statement> opeNameList = wsdlOperationRefs.listProperties(
						opeNameType).toList();
					gp.setOperationName(ExtendedTools.getURIFromStatement(opeNameList.iterator().next()));

					// get the PortType property
					Property portTypeType = model.createProperty(this.propertyManager.__OWLS_GROUNDING_PORTTYPE);
					List<Statement> portTypeList = wsdlOperationRefs.listProperties(
						portTypeType).toList();
					gp.setOperationPortType(ExtendedTools.getURIFromStatement(portTypeList.iterator().next()));

				} else
					return;
			} else
				return;
		} catch (NullPointerException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
			return;
		}
	}

	/**
	 * This method parse all mapping of WsdlInput and store it in the map
	 * IOMapping
	 * 
	 * @param iGrounding
	 *            the individual node, from which the method will gather
	 *            information
	 * @throws ParsingException
	 *             if an error occurs during parsing
	 */
	@SuppressWarnings("unchecked")
	private void gatherInputMapping(Individual iGrounding)
			throws ParsingException {
		// get List of WSDLOperation
		Property inputtype = model.createProperty(this.propertyManager.__OWLS_WSDL_INPUT);
		List<Statement> wsdlInputs = iGrounding.listProperties(inputtype).toList();

		try {
			for (Iterator<Statement> it = wsdlInputs.iterator(); it.hasNext();) {
				// get Individual Corresponding to wsdlOperation
				Statement state = it.next();

				// get Resource (WsdlInputMessageMap) object correspondingh to
				// this Statement (no name define, so, we can't access to the
				// individual directly)
				Resource wsdlInputsMessageMap = state.getResource();

				if (wsdlInputsMessageMap.listProperties().toList().size() > 0) {
					try {
						// get the owlsparameter property
						Property owlsParameterType = model.createProperty(this.propertyManager.__OWLS_GROUNDING_OWLS_PARAMETER);
						List<Statement> owlsParameterList = wsdlInputsMessageMap.listProperties(
							owlsParameterType).toList();
						String owlsParameter = ExtendedTools.getURIFromStatement(owlsParameterList.iterator().next());

						// get the WsdlMessagePart property
						Property wsdlMessagePartType = model.createProperty(this.propertyManager.__OWLS_WSDL_MESSAGE_PART);
						List<Statement> wsdlMessagePartList = wsdlInputsMessageMap.listProperties(
							wsdlMessagePartType).toList();
						String wsdlMessagePart = ExtendedTools.getURIFromStatement(wsdlMessagePartList.iterator().next());

						// add the information in the map of IO_mapping
						this.io_mapping.put(owlsParameter, wsdlMessagePart);

					} catch (NullPointerException e) {
						DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
					}
				}
			}
		} catch (NullPointerException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
			return;
		}
	}

	/**
	 * This method parse all mapping of WsdlOutput and store it in the map
	 * IOMapping
	 * 
	 * @param iGrounding
	 *            the individual node, from which the method will gather
	 *            information
	 * @throws ParsingException
	 *             if an error occurs during parsing
	 */
	@SuppressWarnings("unchecked")
	private void gatherOutputMapping(Individual iGrounding)
			throws ParsingException {
		// get List of WSDLOperation
		Property outputtype = model.createProperty(this.propertyManager.__OWLS_WSDL_OUTPUT);
		List<Statement> wsdlOutputs = iGrounding.listProperties(outputtype).toList();

		try {
			for (Iterator<Statement> it = wsdlOutputs.iterator(); it.hasNext();) {
				// get Individual Corresponding to wsdlOperation
				Statement state = it.next();

				// get Resource (WsdlOutputMessageMap) object correspondingh to
				// this Statement (no name define, so, we can't access to the
				// individual directly)
				Resource wsdlOutputsMessageMap = state.getResource();

				if (wsdlOutputsMessageMap.listProperties().toList().size() > 0) {
					try {
						// get the owlsparameter property
						Property owlsParameterType = model.createProperty(this.propertyManager.__OWLS_GROUNDING_OWLS_PARAMETER);
						List<Statement> owlsParameterList = wsdlOutputsMessageMap.listProperties(
							owlsParameterType).toList();
						String owlsParameter = ExtendedTools.getURIFromStatement(owlsParameterList.iterator().next());

						// get the WsdlMessagePart property
						Property wsdlMessagePartType = model.createProperty(this.propertyManager.__OWLS_WSDL_MESSAGE_PART);
						List<Statement> wsdlMessagePartList = wsdlOutputsMessageMap.listProperties(
							wsdlMessagePartType).toList();
						String wsdlMessagePart = ExtendedTools.getURIFromStatement(wsdlMessagePartList.iterator().next());

						// add the information in the map of IO_mapping
						this.io_mapping.put(owlsParameter, wsdlMessagePart);

					} catch (NullPointerException e) {
						DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
					}
				}
			}
		} catch (NullPointerException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
			return;
		}
	}

	/**
	 * Get the grounding name associated to the name of the Input
	 * 
	 * @param localName :
	 *            the localName of the Input
	 * @return the name of the Input in the WSDL file (grounding), null if this
	 *         name is not know
	 */
	public String getIOGrounding(String localName) {
		return this.io_mapping.get(localName);
	}

	/**
	 * Get the GroundingProperty object associated with the name of the
	 * Converstaion
	 * 
	 * @param conversationURI :
	 *            the name of the Conversation
	 * @return the GroundingProperty object, if the conversaionURI is known,
	 *         null otherwise
	 */
	public GroundingProperty getGroundingProperty(String conversationURI) {
		return this.goundings.get(conversationURI);
	}
}
