package fr.inria.icocoa.servicemodel.parsers.jenaparser;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.PropertyManager;
import fr.inria.icocoa.servicemodel.qos.Level;
import fr.inria.icocoa.servicemodel.qos.Operator;
import fr.inria.icocoa.servicemodel.qos.QoSProperty;
import fr.inria.icocoa.servicemodel.qos.QoSRequirement;
import fr.inria.icocoa.servicemodel.qos.Type;

public class QoSParser {

	/**
	 * The property manager used by this class
	 */
	private PropertyManager propertyManager = PropertyManager.getInstance();

	/**
	 * The current Ontology model
	 */
	private OntModel model;

	public QoSParser(OntModel model) {
		super();
		this.model = model;
	}

	@SuppressWarnings("unchecked")
	public Map<String, List<fr.inria.icocoa.servicemodel.qos.QoSRequirement>> parseFunctionalityLevelRequirements() {

		Map<String, List<QoSRequirement>> result = new HashMap<String, List<QoSRequirement>>();

		Resource functionality = model.getOntResource(propertyManager.__OWLS_CAPABILITY);
		Property hasReq = model.getObjectProperty(propertyManager.__OWLS_QOS_HAS_FUNCTIONALITY_REQUIREMENT);
		Property hasQoSProp = model.getObjectProperty(propertyManager.__OWLS_QOS_HAS_QOS_PROPERTY);
		Property hasValue = model.getObjectProperty(propertyManager.__OWLS_QOS_HAS_VALUE);
		Property hasOperator = model.getObjectProperty(propertyManager.__OWLS_QOS_HAS_OPERATOR);

		List<Individual> funcs = model.listIndividuals(functionality).toList();

		for (Individual func : funcs) {
			if (func.getCardinality(hasReq) > 0) {
				String funcType = func.getRDFType().toString();

				List<QoSRequirement> resultReqs = new ArrayList<QoSRequirement>();
				List<Statement> reqs = func.listProperties(hasReq).toList();

				for (Statement req : reqs) {
					Statement prop = req.getProperty(hasQoSProp);
					OntResource property = (OntResource) prop.getResource().as(
						OntResource.class);

					String propType = property.getRDFType().toString();
					Statement val = prop.getProperty(hasValue);
					double propValue = val.getDouble();

					Statement op = req.getProperty(hasOperator);
					String operator = op.getObject().toString();

					operator = operator.trim();
					Operator newOperator = null;
					if (operator.equals("http://www.dyonisos.org/dyonisos.owl#LessThan")) {
						newOperator = Operator.LESS_THAN;
					} else if (operator.equals("http://www.dyonisos.org/dyonisos.owl#LessThanOrEqualTo")) {
						newOperator = Operator.LESS_THAN_OR_EQUAL_TO;
					} else if (operator.equals("http://www.dyonisos.org/dyonisos.owl#EqualTo")) {
						newOperator = Operator.EQUAL_TO;
					} else if (operator.equals("http://www.dyonisos.org/dyonisos.owl#GreaterThanOrEqualTo")) {
						newOperator = Operator.GREATER_THAN_OR_EQUAL_TO;
					} else if (operator.equals("http://www.dyonisos.org/dyonisos.owl#GreaterThan")) {
						newOperator = Operator.GREATER_THAN;
					} else {
						DebugPrinter.print("Unrecognised operator: "
								+ op.toString() + " in functionality:" + func,
							2);
					}

					propType = propType.trim();
					Type newType = null;
					if (propType.equals("http://www.dyonisos.org/dyonisos.owl#Availability")) {
						newType = Type.AVAILABILITY;
					} else if (propType.equals("http://www.dyonisos.org/dyonisos.owl#Latency")) {
						newType = Type.LATENCY;
					} else {
						DebugPrinter.print("Unrecognised QoS type: " + propType
								+ " in functionality: " + func, 2);
					}

					if (newOperator != null && newType != null) {
						resultReqs.add(new QoSRequirement(new QoSProperty(
								Level.FUNCTIONALITY, newType, propValue),
								newOperator));
					}
				}

				if (!resultReqs.isEmpty()) {
					result.put(funcType, resultReqs);
				}
			}
		}

		return result;
	}

	@SuppressWarnings("unchecked")
	public Map<String, List<QoSProperty>> parseSubFunctionalityLevelProperties() {
		Map<String, List<QoSProperty>> result = new HashMap<String, List<QoSProperty>>();

		Resource functionality = model.getOntResource(propertyManager.__OWLS_CAPABILITY);
		Property hasProp = model.getObjectProperty(propertyManager.__OWLS_QOS_HAS_SUB_FUNCTIONALITY_PROPERTY);
		Property hasValue = model.getObjectProperty(propertyManager.__OWLS_QOS_HAS_VALUE);

		List<Individual> funcs = model.listIndividuals(functionality).toList();
		for (Individual func : funcs) {
			String funcType = func.getRDFType().toString();

			List<QoSProperty> resultProps = new ArrayList<QoSProperty>();
			List<Statement> props = func.listProperties(hasProp).toList();

			for (Statement prop : props) {
				OntResource property = (OntResource) prop.getResource().as(
					OntResource.class);

				String type = property.getRDFType().toString();
				Statement val = prop.getProperty(hasValue);
				double value = val.getDouble();

				type = type.trim();
				Type newType = null;
				if (type.equals("http://www.dyonisos.org/dyonisos.owl#Availability")) {
					newType = Type.AVAILABILITY;
				} else if (type.equals("http://www.dyonisos.org/dyonisos.owl#Latency")) {
					newType = Type.LATENCY;
				} else {
					DebugPrinter.print("Unrecognised QoS type: " + type
							+ " in functionality: " + func, 2);
				}

				if (newType != null) {
					resultProps.add(new QoSProperty(Level.SUB_FUNCTIONALITY,
							newType, value));
				}
			}

			if (!resultProps.isEmpty()) {
				result.put(funcType, resultProps);
			}
		}

		return result;
	}

}
