/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel.parsers.jenaparser;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Set;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.rdf.model.NodeIterator;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.JenaStarter;
import fr.inria.icocoa.common.PropertyManager;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.ControlConstruct;
import fr.inria.icocoa.servicemodel.Input;
import fr.inria.icocoa.servicemodel.Output;
import fr.inria.icocoa.servicemodel.QoSProperty;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.flow.CapabilityFlow;
import fr.inria.icocoa.servicemodel.flow.InvokeCapabilityAction;
import fr.inria.icocoa.servicemodel.parsers.ServiceParser;
import fr.inria.icocoa.servicemodel.parsers.jenaparser.tools.CapabilityForest;
import fr.inria.icocoa.servicemodel.parsers.jenaparser.tools.ExtendedTools;
import fr.inria.icocoa.servicemodel.semantictype.encryptionmanagement.EncodedOntologiesManager;
import fr.inria.icocoa.servicemodel.toolsobjects.Couple;
import fr.inria.icocoa.servicemodel.toolsobjects.ProcessElements;

/**
 * ParserAmigoS2Service.java
 * 
 * If you want parsing will be correct, you need to define Input/Output of
 * capabilities with lang:hasOutput/lang:hasInput with
 * lang="://www.dyonisos.org/dyonisos.owl#" and define Input/Output of Process
 * with process:hasOutput/process:hasInput with
 * process="http://www.daml.org/services/owl-s/1.1/Process.owl#".
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">Sebastien Bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * @author <a href="mailto:Graham.Thomson@inria.fr">Graham Thomson</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 */
public class SDL2ServiceParser extends ServiceParser {

	/**
	 * Treemap to obtain ControlConstruct Object (nameOfControlConstruct ->
	 * ControlConstructObject)
	 */
	private static TreeMap<String, ControlConstruct> association;

	/* Done before creation of class */
	static {
		// creation of the treemap
		SDL2ServiceParser.association = new TreeMap<String, ControlConstruct>();

		// put information in the table
		SDL2ServiceParser.association.put("Any-Order",
				ControlConstruct.ANY_ORDER);
		SDL2ServiceParser.association.put("Choice", ControlConstruct.CHOICE);
		SDL2ServiceParser.association.put("If-Then-Else",
				ControlConstruct.IF_THEN_ELSE);
		SDL2ServiceParser.association.put("Iterate", ControlConstruct.ITERATE);
		SDL2ServiceParser.association.put("Repeat-Until",
				ControlConstruct.REPEAT_UNTIL);
		SDL2ServiceParser.association.put("Repeat-While",
				ControlConstruct.REPEAT_WHILE);
		SDL2ServiceParser.association
				.put("Sequence", ControlConstruct.SEQUENCE);
		SDL2ServiceParser.association.put("Split", ControlConstruct.SPLIT);
		SDL2ServiceParser.association.put("Split-Join",
				ControlConstruct.SPLIT_JOIN);
	}

	/**
	 * Contains the association (URIOfCapability, CapabilityObject)
	 */
	private TreeMap<String, Capability> mapping = new TreeMap<String, Capability>();

	/**
	 * Contains the association (URIOfProcess,isItAtomic). If the Boolean is
	 * true, then the process is Atomic.
	 */
	private TreeMap<String, Boolean> storeAtomiticyOfProcess = new TreeMap<String, Boolean>();

	/**
	 * The property manager used by this class
	 */
	private PropertyManager propertyManager = PropertyManager.getInstance();

	// private OntDocumentManager manager = OntDocumentManager.getInstance();

	/**
	 * The current OntologyModel used for parsing
	 */
	private OntModel model = null;

	/**
	 * Parser for the process
	 */
	private TaskParser taskParser = null;

	/**
	 * Parser for the grounding
	 */
	private GroundingParser groundingParser = null;

	
	/**
	 * the hashCode corresponding to the list of import of the last file read
	 */
	private String importsHashCode =  null;

	/**
	 * The list of URI of the imported ontologies of the last file read
	 */
	private List<String> listOfImports;

	/**
	 * Constructor
	 */
	public SDL2ServiceParser() {

		ServiceParser.currentParser = this;

		// initialisation of the JenaParser -> the reasoner must not be use,
		// because it cause a "memory overhead" problem
		model = JenaStarter.getInstanceOfOntModel(false);
	}

	/**
	 * Constructor
	 * 
	 * @param getGrounding :
	 *            put the value to "true" if you want need to get the grounding,
	 *            "false" otherwise
	 * 
	 * @param uri :
	 *            the URI of the file we have to parse
	 * @throws ParsingException
	 */
	@SuppressWarnings("unchecked")
	private void initialiseParsing(String uri, boolean getGrounding)
			throws ParsingException {

		// read the file
		this.model.read(uri, null);
		
		/*
		 * Collect the closure of imports done for the description
		 * and update the importHashCode
		 */
		//Collect the closure of import
		Set<String> subModels = (Set<String>) this.model
				.listImportedOntologyURIs(true);
		List<String> listOfImports = new ArrayList<String>(subModels);
		
		//create the importHashCode
		if (listOfImports == null)
			this.importsHashCode = null;
		else {
			String result = "";

			// sort all imports (alphabetical order)
			TreeMap<String, String> listInOrder = new TreeMap<String, String>();
			for (String s : listOfImports)
				listInOrder.put(s, s);

			//initialise listOfImports
			this.listOfImports = new ArrayList<String>();
			
			// create a hashcode (corresponding to aggregation of the String)
			Iterator<String> it = listInOrder.keySet().iterator();
			while (it.hasNext()){
				String url = it.next().trim();				
				result += url+EncodedOntologiesManager.IMPORTED_ONTOLOGIES_HASHCODE_SEPARATOR;

				//update the list of imports
				this.listOfImports.add(url);
			}			
			
			//update the hashcode
			this.importsHashCode = result;			
		}
		

		// initialisation of the parser for Process
		taskParser = new TaskParser(this.model, this);

		// initialisation of the parser for Grounding if the grounding is
		// required
		if (getGrounding)
			this.groundingParser = new GroundingParser(this.model);

		// initialisation of this object
		this.quickCreationOfCapabilities();// reference knowed capabilities
		this.quickVisionOfAtomicityOfProcesses();// reference knowed
		// processes

		

		// Diplay Result of Initialisation
		// this.displayKnowedCapabilities();
		// this.displayKnowedProcess();

	}

	/**
	 * Get the list of all processes
	 * (http://www.daml.org/services/owl-s/1.1/Process.owl#Process)
	 * 
	 * @return the list of Individual corresponding to Process Class
	 */
	@SuppressWarnings("unchecked")
	private List<Individual> getAllProcesses() {

		Resource processRes = model
				.getOntResource(this.propertyManager.__OWLS_PROCESS);

		List<Individual> lprocess = model.listIndividuals(processRes).toList();

		return lprocess;

	}

	/**
	 * Get the list of all atomic processes
	 * (http://www.daml.org/services/owl-s/1.1/Process.owl#AtomicProcess)
	 * 
	 * @return the list of Individual corresponding to AtomicProcess Class
	 */
	@SuppressWarnings("unchecked")
	private List<Individual> getAllAtomicProcesses() {

		Resource processRes = model
				.getOntResource(this.propertyManager.__OWLS_ATOMIC_PROCESS);

		List<Individual> lprocess = model.listIndividuals(processRes).toList();

		return lprocess;
	}

	/**
	 * Get the list of all composite processes
	 * (http://www.daml.org/services/owl-s/1.1/Service.owl#Service)
	 * 
	 * @return the list of Individual corresponding to CompositeProcess Class
	 */
	@SuppressWarnings("unchecked")
	private List<Individual> getAllCompositeProcesses() {

		Resource processRes = model
				.getOntResource(this.propertyManager.__OWLS_COMPOSITE_PROCESS);

		List<Individual> lprocess = model.listIndividuals(processRes).toList();

		return lprocess;

	}

	/**
	 * Get the list of all capabilities of the Service (given par field
	 * "hasProvidedCapability" or "hasCapability"
	 * (http://www.daml.org/services/owl-s/1.1/Process.owl#CompositeProcess)
	 * 
	 * @return the list of Individual corresponding to CompositeProcess Class
	 * @throws ParsingException
	 */
	@SuppressWarnings("unchecked")
	private List<Individual> getAllCapabilities() throws ParsingException {

		// DebugPrinter.print("pm =\n "+this.propertyManager.myToString(),
		// DebugPrinter.INFO);
		// DebugPrinter.print("this.propertyManager.__OWLS_CAPABILITY =
		// "+this.propertyManager.__OWLS_CAPABILITY, DebugPrinter.INFO);

		Resource capaRes = model
				.getOntResource(this.propertyManager.__OWLS_CAPABILITY);
		if (capaRes == null) {
			throw new ParsingException("The class "
					+ this.propertyManager.__OWLS_CAPABILITY
					+ " has not been found");
		}

		ExtendedIterator itr = model.listIndividuals(capaRes);
		if (itr == null) {
			System.out.println("nought for " + capaRes);
		}
		List<Individual> lcapa = itr.toList();

		// DebugPrinter.print(lcapa, "all capas", 1);

		List<Individual> result = new ArrayList<Individual>();
		for (Individual capability : lcapa)
			result.add(capability);

		// QoS
		/*
		 * for (Individual capability : lcapa) { //
		 * DebugPrinter.print(capability.listProperties().toList(),"list of
		 * properties of " + capability, 1);
		 * 
		 * List<Statement> props = capability.listProperties().toList();
		 * boolean isQoSFeature = true; for (Statement prop : props) { String s =
		 * prop.getObject().toString(); // TODO Fix this dirty, dirty hack -
		 * under current parsing scheme, any QoS feature, regardless of the //
		 * syntax used to declare it, will be recognised as having type
		 * Functionality, and a Functnioality will // be recognised // as having
		 * type QoSProperty / QoSRequirement. However, real Functionalities are
		 * explicitly declared as // being subclasses of owl:Thing, while QoS
		 * features are not. As the current parsing scheme uses the // default //
		 * of no reasoner, this relation is not implied for QoS features, and so
		 * can be used to identify them. // (likewise for setha:Entity) if
		 * (s.equals("http://www.w3.org/2002/07/owl#Thing") ||
		 * s.equals("http://www.thalesgroup.com/setha-dyonisos-demo.owl#Entity")) {
		 * isQoSFeature = false; break; } } if (!isQoSFeature) {
		 * result.add(capability); // DebugPrinter.print("Including non-QoS
		 * feature: " + capability, 1); } // else { //
		 * DebugPrinter.print("Ignoring QoS feature: " + capability, 1); // } }
		 */

		return result;
	}

	/**
	 * Get the list of all Service Profile
	 * 
	 * 
	 * @return the list of Individual corresponding to CompositeProcess Class
	 */
	@SuppressWarnings("unchecked")
	private List<Individual> getAllServicesProfile() {
		Resource capaRes = model
				.getOntResource(this.propertyManager.__OWLS_SERVICEPROFILE);

		List<Individual> lcapa = model.listIndividuals(capaRes).toList();

		return lcapa;
	}

	/**
	 * Create the capacity object, without filling the different field. The
	 * created capabilities will be added in the Map mapping
	 * 
	 * @param capability :
	 *            the Individual object WHICH IS a capability
	 */
	private void quickCreationOfCapability(Individual capability) {

		// DebugPrinter.print("capability = "+capability.getRDFType(), 1);

		// get the type of capability
		String type = capability.getRDFType().toString();

		// creation of the resulting capability with its name
		Capability result = new Capability(type, capability.getLocalName());

		this.mapping.put(capability.getURI(), result);
	}

	/**
	 * Convert a capability URI into a Capability object
	 * 
	 * @param capability
	 *            an Individaul WHICH IS a capability
	 * @return the modified Capability, or null if the individual is not a
	 *         Capability
	 * @exception ParsingException
	 *                if an error occurs during the split process (see message
	 *                field)
	 */
	@SuppressWarnings("unchecked")
	private Capability convertToSubCapability(Individual capability)
			throws ParsingException {
		try {

			String uri = capability.getURI();

			// get the created capability from the map thanks to is name.
			Capability resultCapability = this.getCapability(uri);

			// set the URI of the Capability
			resultCapability.setCapabilityURI(uri);

			// get conversations associated with
			Property type = model
					.createProperty(this.propertyManager.__HASCONVERSATION_PROPERTY);
			List<Statement> lConversation = capability.listProperties(type)
					.toList();

			Individual _process = null;
			for (int i = 0; i < lConversation.size(); i++) {

				// get associated individual
				_process = ExtendedTools.getIndividualFromStatement(model,
						lConversation.get(i));

				// extract subcapabilities and Control structure
				this.setConversation(_process, resultCapability);

				// extract inputs
				this.setInputs(_process, resultCapability);

				// extract outputs
				this.setOutputs(_process, resultCapability);

			}

			// set if the capability is atomic or not
			// DebugPrinter.print(this.storeAtomiticyOfProcess,"this.storeAtomiticyOfProcess",1);
			resultCapability.setAtomic(this.storeAtomiticyOfProcess.get(
					resultCapability.getConversationURI()).booleanValue());

			// add the grounding id required
			if (this.groundingParser != null)
				resultCapability.setGrounding(this.groundingParser
						.getGroundingProperty(resultCapability
								.getConversationURI()));

			return resultCapability;
		} catch (NullPointerException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
			if (capability != null) {
				throw new ParsingException("The Individual named "
						+ capability.getLocalName()
						+ " is not recognize as a capability. (More : "
						+ e.getMessage() + ")");
			} else {
				throw new ParsingException(
						"The called Individual is null. (More : "
								+ e.getMessage() + ")");
			}
		} catch (ParsingException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
			throw new ParsingException(
					"An error occurs during the parsing of the capability "
							+ capability.getLocalName() + ". (More : "
							+ e.getMessage() + ")");
		} catch (NoSuchElementException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
			throw new ParsingException("The Individual named "
					+ capability.getLocalName() + " is not recognize. (More : "
					+ e.getMessage() + ")");
		}

	}

	/**
	 * Convert a Individual Object into an Input object
	 * 
	 * @param i_input
	 *            an individual ID WHICH IS an intput
	 * @return the Input containing information of the individual
	 * @throws ParsingException
	 *             if an error occurs during parsing
	 */
	private Input convertToInput(Individual i_input) throws ParsingException {
		try {

			// get the statement object for the property "ParameterType". This
			// contains "ontology#Name Type"
			Property type = model
					.createProperty(this.propertyManager.__PARAMETERTYPE_PROPERTY);
			Statement state = i_input.listProperties(type).nextStatement();

			// get only "ontology#Name"
			String ontologyAndName = ExtendedTools.getURIFromStatement(state);

			// split ontology and name
			String[] ontologyAndName_splitted = ontologyAndName.split("#");

			// if you don't put a space after the declaration of the class of
			// Input/Output, a ^^nameOfCodage appear.
			// we need to remove it !
			/*
			 * if(ontologyAndName_splitted[1].contains("^^")){ StringTokenizer
			 * st = new StringTokenizer(ontologyAndName_splitted[1],"^^");
			 * ontologyAndName_splitted[1] = st.nextToken(); }
			 */

			// the last parameter is the localName of the Input (change to
			// getURI if necessary)
			return new Input(ontologyAndName_splitted[1],
					ontologyAndName_splitted[0], state.getSubject().getURI());
		} catch (IndexOutOfBoundsException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
			if (i_input != null) {
				throw new ParsingException("The Individual named "
						+ i_input.getLocalName()
						+ " is not recognize as an Input. (More : "
						+ e.getMessage() + ")");
			} else {
				throw new ParsingException(
						"The called Individual is null. (More : "
								+ e.getMessage() + ")");
			}
		} catch (NullPointerException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
			if (i_input != null) {
				throw new ParsingException("The Individual named "
						+ i_input.getLocalName()
						+ " is not recognize as an Input. (More : "
						+ e.getMessage() + ")");
			} else {
				throw new ParsingException(
						"The called Individual is null. (More : "
								+ e.getMessage() + ")");
			}
		} catch (NoSuchElementException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
			if (i_input != null) {
				throw new ParsingException("The Individual named "
						+ i_input.getLocalName()
						+ " is not recognize. (More : " + e.getMessage() + ")");
			} else {
				throw new ParsingException(
						"The called Individual is null. (More : "
								+ e.getMessage() + ")");
			}
		}
	}

	/**
	 * Convert a Individual Object into an Output object
	 * 
	 * @param i_output
	 *            an individual ID WHICH IS an output
	 * @return the Output containing information of the individual
	 * 
	 * @throws ParsingException
	 *             if an error occurs during parsing
	 */
	private Output convertToOutput(Individual i_output) throws ParsingException {
		try {
			// System.out.println("SDL2SERvice::convertToOutput() --
			// individual="+i_output);

			// get the statement object for the property "ParameterType". This
			// contains "ontology#Name Type"
			Property type = model
					.createProperty(this.propertyManager.__PARAMETERTYPE_PROPERTY);
			Statement state = i_output.listProperties(type).nextStatement();

			// get only "ontology#Name"
			String ontologyAndName = ExtendedTools.getURIFromStatement(state);

			// split ontology and name
			String[] ontologyAndName_splitted = ontologyAndName.split("#");

			// if you don't put a space after the declaration of the class of
			// Input/Output, a ^^nameOfCodage appear.
			// we need to remove it !
			/*
			 * if(ontologyAndName_splitted[1].contains("^^")){ StringTokenizer
			 * st = new StringTokenizer(ontologyAndName_splitted[1],"^^");
			 * ontologyAndName_splitted[1] = st.nextToken(); }
			 */

			// the last parameter is the localName of the Output (change to
			// getURI if necessary)
			return new Output(ontologyAndName_splitted[1],
					ontologyAndName_splitted[0], state.getSubject().getURI());
		} catch (IndexOutOfBoundsException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
			if (i_output != null) {
				throw new ParsingException("The Individual named "
						+ i_output.getLocalName()
						+ " is not recognize as an Output. (More : "
						+ e.getMessage() + ")");
			} else {
				throw new ParsingException(
						"The called Individual is null. (More : "
								+ e.getMessage() + ")");
			}
		} catch (NullPointerException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
			if (i_output != null) {

				throw new ParsingException("The Individual named "
						+ i_output.getLocalName()
						+ " is not recognize as an Output. (More : "
						+ e.getMessage() + ")");
			} else {
				throw new ParsingException(
						"The called Individual is null. (More : "
								+ e.getMessage() + ")");
			}
		} catch (NoSuchElementException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
			if (i_output != null) {

				throw new ParsingException("The Individual named "
						+ i_output.getLocalName()
						+ " is not recognize. (More : " + e.getMessage() + ")");
			} else {
				throw new ParsingException(
						"The called Individual is null. (More : "
								+ e.getMessage() + ")");
			}
		}
	}

	/**
	 * This method find the ControlConstruct object associated to the control
	 * Strin,g
	 * 
	 * @param control
	 *            the string coding the ControlContruct
	 * @return the ContrcolConstruc associated to the string, null if not found
	 */
	private static ControlConstruct getControlStructure(String control) {
		return association.get(control);
	}

	/**
	 * This method search all process and convert them into Capalilities
	 * whithout filling fields. the created capability are put into the map
	 * "mapping".
	 * 
	 * @throws ParsingException
	 * 
	 */
	private void quickCreationOfCapabilities() throws ParsingException {
		List<Individual> list = this.getAllCapabilities();
		for (int i = 0; i < list.size(); i++) {
			this.quickCreationOfCapability(list.get(i));
		}

	}

	/**
	 * This method search all process and store them URI in the map
	 * "storeAtomiticyOfProcess", in association with a boolean that indicts if
	 * the process is Atomic (boolean = tue) or not (boolean = false).
	 * 
	 */
	private void quickVisionOfAtomicityOfProcesses() {
		// store AtomicProcess
		List<Individual> list = this.getAllAtomicProcesses();
		for (int i = 0; i < list.size(); i++) {
			this.storeAtomiticyOfProcess
					.put(list.get(i).getURI(), Boolean.TRUE);
		}

		// store CompositeProcess
		list = this.getAllCompositeProcesses();
		for (int i = 0; i < list.size(); i++) {
			this.storeAtomiticyOfProcess.put(list.get(i).getURI(),
					Boolean.FALSE);
		}

	}

	/**
	 * This methode retutn the Capability associated the given URI
	 * 
	 * @param capa_URI
	 *            the URI of the Capability we want
	 * @return the whiched capability, null if the URI doen't correspond with
	 *         any capability (but process or other)
	 */
	private Capability getCapability(String capa_URI) {
		return this.mapping.get(capa_URI);
	}

	/**
	 * This method diplay all URI ok knowed Capability object (key of map
	 * "mapping")
	 * 
	 */
	private void displayKnowedCapabilities() {
		ArrayList<String> keys = new ArrayList<String>(this.mapping.keySet());
		System.out.println("Knowed URI of capabilities");
		for (int i = 0; i < keys.size(); i++) {
			System.out.println("URI nb " + i + " : " + keys.get(i));
		}

	}

	/**
	 * This method diplay all URI ok knowed Process object (key of map
	 * "storeAtomiticyOfProcess")
	 * 
	 */
	private void displayKnowedProcess() {
		ArrayList<String> keys = new ArrayList<String>(
				this.storeAtomiticyOfProcess.keySet());
		System.out.println("Knowed URI of Process");
		for (int i = 0; i < keys.size(); i++) {
			System.out.println("URI nb " + i + " : " + keys.get(i));
		}

	}

	/**
	 * This method diplay all URI ok knowed Capability object (key of map
	 * "mapping")
	 * 
	 */
	private void displayKnowedCapabilitiesComplete() {
		ArrayList<String> keys = new ArrayList<String>(this.mapping.keySet());
		System.out.println("Knowed capabilities");
		for (int i = 0; i < keys.size(); i++) {
			System.out.println(this.mapping.get(keys.get(i)));
		}

	}

	/**
	 * This method test, if the Individual is a process, if it is an Atomic
	 * process
	 * 
	 * @param process :
	 *            the individual WHICH IS a Process
	 * @return true if the Process is an AtomicProcess, false otherwise
	 * @throws ParsingException
	 *             if the Individual is not a processs
	 */
	private boolean isAtomic(Individual process) throws ParsingException {
		// get the value from the map.
		Boolean result = this.storeAtomiticyOfProcess.get(process.getURI());
		if (result == null)
			throw new ParsingException("The individual " + process
					+ " isn't a Process");
		return result.booleanValue();
	}

	/**
	 * This method extract subCapabilities from a process
	 * 
	 * @param process :
	 *            an Individual WHICH IS a process
	 * @param resultCapability :
	 *            the Capability we want to complete
	 * @throws ParsingException
	 *             if the Individual is not a process
	 */
	private void setConversation(Individual process, Capability resultCapability)
			throws ParsingException {

		resultCapability.setConversationURI(process.getURI());

		// if the process is atomic, we need to find subcapabilities
		if (!this.isAtomic(process)) {

			Couple<ProcessElements, CapabilityFlow> taskParsing;
			try {
				taskParsing = taskParser.getFlowAndProcess(process.getURI());
			} catch (CloneNotSupportedException e) {
				DebugPrinter.print(e, DebugPrinter.ERROR);
				throw new ParsingException("An error occurs during parsing. ("
						+ e.getLocalizedMessage() + ")");
			}

			// add composite flow
			resultCapability.setFlow(taskParsing.getSecond());

			// parse this process
			ProcessElements _processElement = taskParsing.getFirst();

			// put the SubCapabililties
			Capability _capability = null;
			for (int j = 0; j < _processElement.size(); j++) {
				// If the Capability isn't in the map, so it's not a Capability
				// so
				// we ignore it
				_capability = this.getCapability(_processElement.getElement(j)
						.getCapability());

				if (_capability != null) {
					resultCapability.addSubCapability(_capability);
				}
			}

			// Put the ControlStructure only if there is subcapabilities
			if (resultCapability.getSubCapabilities().size() > 0) {
				resultCapability.setCC(SDL2ServiceParser
						.getControlStructure(_processElement.getControl()));
			}

		} else {
			// create the Atomic invokeAction object
			resultCapability.setFlow(new InvokeCapabilityAction(
					resultCapability));
		}

	}

	/**
	 * This method extract Output from a process
	 * 
	 * @param process :
	 *            an Individual WHICH IS a process
	 * @param resultCapability :
	 *            the Capability we want to complete
	 * @throws ParsingException
	 *             if the Individual is not a process
	 */
	@SuppressWarnings("unchecked")
	private void setOutputs(Individual process, Capability resultCapability)
			throws ParsingException {
		// get all outputs
		Property type = model
				.createProperty(this.propertyManager.__HASOUTPUT_PROCESS_PROPERTY);
		List<Statement> lOutput = process.listProperties(type).toList();
		Output _output = null;
		for (int i = 0; i < lOutput.size(); i++) {
			_output = this.convertToOutput(ExtendedTools
					.getIndividualFromStatement(model, lOutput.get(i)));
			resultCapability.addOutput(_output);
			_output.setParentCapability(resultCapability);

			// get the grounding if required
			if (this.groundingParser != null)
				_output.setGroundingValue(this.groundingParser
						.getIOGrounding(_output.getLocalName()));
		}
	}

	/**
	 * This method extract Input from a process
	 * 
	 * @param process :
	 *            an Individual WHICH IS a process
	 * @param resultCapability :
	 *            the Capability we want to complete
	 * @throws ParsingException
	 *             if the Individual is not a process
	 */
	@SuppressWarnings("unchecked")
	private void setInputs(Individual process, Capability resultCapability)
			throws ParsingException {
		// get all inputs
		Property type = model
				.createProperty(this.propertyManager.__HASINPUT_PROCESS_PROPERTY);
		List<Statement> lInput = process.listProperties(type).toList();
		Input _input = null;
		for (int i = 0; i < lInput.size(); i++) {
			_input = this.convertToInput(ExtendedTools
					.getIndividualFromStatement(model, lInput.get(i)));

			// get the grounding if required
			if (this.groundingParser != null)
				_input.setGroundingValue(this.groundingParser
						.getIOGrounding(_input.getLocalName()));

			_input.setParentCapability(resultCapability);
			resultCapability.addInput(_input);
		}

	}

	/**
	 * This method extract all provided capabilities from the service profile
	 * 
	 * @param serviceProfile
	 *            an Individual WHICH IS a ServiceProfile
	 * @return the List of Capability
	 * @throws ParsingException
	 *             if an error occurs during parsing
	 */
	@SuppressWarnings("unchecked")
	private ArrayList<Capability> getProvidedCapability(
			Individual serviceProfile) throws ParsingException {
		// get all the providedCapability
		Property type = model
				.createProperty(this.propertyManager.__HASPROVIDEDCAPABILITY_PROPERTY);
		List<Statement> lcapa = serviceProfile.listProperties(type).toList();

		// creation of the result list
		ArrayList<Capability> result = new ArrayList<Capability>();
		Capability _capa = null;
		for (int i = 0; i < lcapa.size(); i++) {
			_capa = this.getCapability(ExtendedTools.getURIFromStatement(lcapa
					.get(i)));
			if (_capa != null)
				result.add(_capa);
		}

		return result;
	}

	/**
	 * This method extract all capabilities from the service profile
	 * 
	 * @param serviceProfile
	 *            an Individual WHICH IS a ServiceProfile
	 * @return the List of Capability
	 * @throws ParsingException
	 *             if an error occurs during parsing
	 */
	@SuppressWarnings("unchecked")
	private ArrayList<Capability> getCapability(Individual serviceProfile)
			throws ParsingException {
		// get all the providedCapability
		Property type = model
				.createProperty(this.propertyManager.__HASCAPABILITY_PROPERTY);
		List<Statement> lcapa = serviceProfile.listProperties(type).toList();

		// creation of the result list
		ArrayList<Capability> result = new ArrayList<Capability>();
		Capability _capa = null;
		for (int i = 0; i < lcapa.size(); i++) {
			_capa = this.getCapability(ExtendedTools.getURIFromStatement(lcapa
					.get(i)));
			if (_capa != null)
				result.add(_capa);
		}

		return result;
	}

	/**
	 * This method extract all required capabilities from the service profile
	 * 
	 * @param serviceProfile
	 *            an Individual WHICH IS a ServiceProfile
	 * @return the List of Capability
	 * @throws ParsingException
	 *             if an error occurs during parsing
	 */
	@SuppressWarnings("unchecked")
	private ArrayList<Capability> getRequiredCapability(
			Individual serviceProfile) throws ParsingException {
		// get all the providedCapability
		Property type = model
				.createProperty(this.propertyManager.__HASREQUIREDCAPABILITY_PROPERTY);
		List<Statement> lcapa = serviceProfile.listProperties(type).toList();

		// creation of the result list
		ArrayList<Capability> result = new ArrayList<Capability>();
		Capability _capa = null;
		for (int i = 0; i < lcapa.size(); i++) {
			_capa = this.getCapability(ExtendedTools.getURIFromStatement(lcapa
					.get(i)));
			if (_capa != null)
				result.add(_capa);
		}

		return result;
	}

	/**
	 * This method extract name of from the service profile
	 * 
	 * @param serviceProfile
	 *            an Individual WHICH IS a ServiceProfile
	 * @return the name of the service
	 * @throws ParsingException
	 *             if an error occurs during parsing
	 */
	@SuppressWarnings("unchecked")
	private String getServiceName(Individual serviceProfile)
			throws ParsingException {
		// get all the providedCapability
		Property type = model
				.createProperty(this.propertyManager.__PRESENTEDBY_PROPERTY);
		List<Statement> lname = serviceProfile.listProperties(type).toList();
		try {
			// there is only one name
			String URI = ExtendedTools.getURIFromStatement(lname.get(0));

			return URI.split("#")[1];

		} catch (IndexOutOfBoundsException e) {
			throw new ParsingException("The Individual named "
					+ serviceProfile.getLocalName()
					+ " has an incorrect \"presentedBy\" field. (More : "
					+ e.getMessage() + ")");
		} catch (NullPointerException e) {
			throw new ParsingException("The Individual named "
					+ serviceProfile.getLocalName()
					+ " is not recognize as an ServiceProfile. (More : "
					+ e.getMessage() + ")");
		}
	}

	/**
	 * This method extract the Service corresponding to the Individual
	 * 
	 * @param service
	 * @return
	 * @throws ParsingException
	 *             if an error occurs during parsing
	 */
	private Service convertToService(Individual service)
			throws ParsingException {
		// creation of the Service object
		Service result = new Service();

		// the capability forest, for databinding
		CapabilityForest capabilityForest = new CapabilityForest();

		// set name
		result.setLocalName(this.getServiceName(service));

		// set base
		result.setBase(this.getServiceBase(service));

		// set type
		result.setType(this.getServiceType(service));

		// get RequiredCapabilities
		{
			ArrayList<Capability> tmpList = this.getRequiredCapability(service);
			Capability _capa = null;
			for (int i = 0; i < tmpList.size(); i++) {

				// we get the capa for the tmp ArrayList
				_capa = tmpList.get(i);

				// we add Service field
				_capa.setService(result);

				// we add the automaton for composition
				// TODO Fully seperate concerns - move all composition aspects
				// to that project,
				// leaving only repository aspects in this project.
				// _capa.setAutomaton(ServiceModel2Automata.generate_One_Level_Automaton(_capa));

				// we add the capability into the RequiredCapability List of
				// result
				result.addRequiredCapability(_capa);

				// we add the Capability in the tree of Capabilities
				capabilityForest.addCapability(_capa);
			}
		}

		// get ProvidedCapabilities
		{
			ArrayList<Capability> tmpList = this.getProvidedCapability(service);
			Capability _capa = null;
			for (int i = 0; i < tmpList.size(); i++) {
				// we get the capa for the tmp ArrayList
				_capa = tmpList.get(i);

				// we add Service field
				_capa.setService(result);

				// we add the automaton for composition
				// TODO Fully seperate concerns - move all composition aspects
				// to that project,
				// leaving only repository aspects in this project.
				// _capa.setAutomaton(ServiceModel2Automata.generate_One_Level_Automaton(_capa));

				// change state of Input/outputs to reqired
				_capa.setIOAsProvided(true);

				// we add the capability into the ProvidedCapability List of
				// result
				result.addProvidedCapability(_capa);

				// we add the Capability in the tree of Capabilities
				capabilityForest.addCapability(_capa);
			}
		}

		// get capabilities
		{
			ArrayList<Capability> tmpList = this.getCapability(service);
			Capability _capa = null;
			for (int i = 0; i < tmpList.size(); i++) {
				// we get the capa for the tmp ArrayList
				_capa = tmpList.get(i);

				// we add Service field
				_capa.setService(result);

				// we add the automaton for composition
				// TODO Fully seperate concerns - move all composition aspects
				// to that project,
				// leaving only repository aspects in this project.
				// _capa.setAutomaton(ServiceModel2Automata.generate_One_Level_Automaton(_capa));
				// we add the capability into the Capability List of result
				result.addCapability(_capa);
			}
		}

		// set the map of BindedVariables
		result.setBinded_variable(taskParser.getBinded_variable());

		// remove all FuturCapabilityFlow object to get a completly parsed
		// workflow
		try {
			CapabilityFlow.removeFuturObject(result);
		} catch (ParsingException e) {
			DebugPrinter.print(e, DebugPrinter.ERROR);
			throw e;
		}
		return result;
	}

	/**
	 * 
	 * SDL2ServiceParser.java<br>
	 * THis class define an handler so as to gather the text between a specified
	 * tag.
	 * 
	 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> -
	 *         <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
	 * 
	 * Creation Date : Jun 12, 2007 5:26:58 PM
	 */
	private class MyHandler extends DefaultHandler {

		/** Indicate if you had to gather the character */
		private boolean gatherEnable = false;

		/** The text to gather */
		private String gathered = "";

		/** the tag to find */
		private String tag = "";

		/** the current String */
		private String current = "";

		/** the current level of indent */
		private int indentlevel = 0;

		/** the indent sting */
		private static final String INDENTSTRING = " ";

		@Override
		public void startElement(String uri, String localName, String qName,
				Attributes attributes) throws SAXException {
			super.startElement(uri, localName, qName, attributes);

			// when the required tag is found, start to gather the data
			if (qName.equals(tag) && !gatherEnable)
				gatherEnable = true;
			if (gatherEnable) {
				// if gathering enabled, then copy the discovered tag and
				// related properties
				this.gathered += this.getPrefix() + "<" + qName;
				for (int i = 0; i < attributes.getLength(); i++) {
					this.gathered += " " + attributes.getQName(i) + "=\""
							+ attributes.getValue(i) + "\"";
				}
				this.gathered += ">\n";
				this.indentlevel++;
			}

		}

		@Override
		public void characters(char[] ch, int start, int length)
				throws SAXException {
			super.characters(ch, start, length);
			// store the values contained between the two last tag
			this.current = (new String(ch, start, length)).trim();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.xml.sax.helpers.DefaultHandler#endElement(java.lang.String,
		 *      java.lang.String, java.lang.String)
		 */
		@Override
		public void endElement(String uri, String localName, String qName)
				throws SAXException {
			super.endElement(uri, localName, qName);
			// DebugPrinter.print("uri="+uri+" localname="+localName+"
			// qName="+qName, 1);

			// if gathering enable, copy the closed tag and the data contained
			// between this tag and the relative open tag (= this.current)
			if (gatherEnable) {
				if (!current.equals(""))
					this.gathered += this.getPrefix() + current + "\n";

				this.indentlevel--;
				this.gathered += this.getPrefix() + "</" + qName + ">\n";

				// if the closed tag relative to the required tag is found, then
				// stop the gathering of information
				if (qName.equals(tag)) {
					gatherEnable = false;
				}
			}
		}

		/**
		 * Get a prefix of each line so as to obtain a correct indentation
		 * 
		 * @return a String corresponding to number of space so as to have a
		 *         correct indentation
		 */
		private String getPrefix() {
			String result = "";
			for (int i = 0; i < this.indentlevel; i++)
				result += INDENTSTRING;
			return result;
		}

		/**
		 * @return the gathered
		 */
		public String getGathered() {
			return gathered;
		}

		/**
		 * @param tag
		 *            the tag to set
		 */
		public void setTag(String tag) {
			this.tag = tag;
		}

	}

	/**
	 * Get the QoS Part of the description
	 * 
	 * @param uri :
	 *            the uri of the file containing the QoS definition (between tag
	 *            "QoSProfile")
	 * @return the QoSProperty Object, <i>null</i> if no QAoSProperty has been
	 *         defined.
	 */
	private QoSProperty getQoSProperty(String uri) {
		Resource qosProfileRes = model
				.getOntResource(this.propertyManager.__QOSPROFILE);

		Individual qosIndi = model
				.getIndividual(this.propertyManager.__QOSPROFILE);

		// if there is some QoSProfile
		if (qosIndi != null) {

			// get the name of the tag containing the QoSProfile def
			String tag = model.getNsURIPrefix(qosProfileRes.getNameSpace())
					+ ":QoSProfile";

			String result = "";

			// Use a sax parser to extract the information
			XMLReader myReader;
			try {
				myReader = XMLReaderFactory.createXMLReader();

				// creation of the SAX handler
				MyHandler handler = new MyHandler();

				// add the search tag in the handler
				handler.setTag(tag);

				// put the handler in the sax machine
				myReader.setContentHandler(handler);
				myReader.setErrorHandler(handler);

				// parse the file
				myReader.parse(uri);

				// get the QosProfil part of code (test if not empty)
				if (!handler.getGathered().equals(""))
					result += handler.getGathered() + "\n";
				else
					return null;

			} catch (SAXException e) {
				DebugPrinter.print(e.getStackTrace(), e.getMessage(), 4);
				return null;
			} catch (IOException e) {
				DebugPrinter.print(e.getStackTrace(), e.getMessage(), 4);
				return null;
			}

			// DebugPrinter.print(model.getNsPrefixMap(),"prefix",1);
			/*
			 * ExtendedIterator it = model.listSubModels(); while(it.hasNext())
			 * System.out.println(((OntModel)it.next()).getNsPrefixURI(""));
			 */
			/*
			 * Set s = model.listImportedOntologyURIs(); Iterator it =
			 * s.iterator(); while(it.hasNext()){ System.out.println(it.next()); }
			 */

			return new QoSProperty(result);

		} else
			return null;
	}

	/**
	 * Parse the file to get the "base" of service
	 * 
	 * @param service :
	 *            the individual object coding the Base
	 * @return the base of the service
	 */
	private String getServiceBase(Individual service) {
		try {
			return service.listProperties().nextStatement().asTriple()
					.getSubject().toString().split("#")[0];
		} catch (NullPointerException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
		} catch (IndexOutOfBoundsException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
		}
		return null;
	}

	/**
	 * Parse the type to get the "type" of the service
	 * 
	 * @param service :
	 *            the individual object coding the service
	 * @return the type of the service if found, <i>null</i> otherwise
	 */
	private String getServiceType(Individual service) {
		NodeIterator nitr = model.listObjectsOfProperty(model
				.getProperty(propertyManager.__SERVICE_TYPE));
		if (nitr.hasNext()) {
			RDFNode serviceTypeNode = nitr.nextNode();
			return serviceTypeNode.toString().split("\\^\\^")[0].trim();
		}
		return null;
	}

	/**
	 * THis method required the parsing of the grounding for the service. <br>
	 * If the parsing is not required (getGrounding in constructor = false), do
	 * nothing.
	 * 
	 * @throws ParsingException
	 *             if an error occurs during parsing
	 */
	private void parseGrounding() throws ParsingException {
		if (this.groundingParser != null)
			this.groundingParser.parseService();
	}

	/**
	 * @return the mapping
	 */
	TreeMap<String, Capability> getMapping() {
		return mapping;
	}

	/**
	 * Get the name of the URI of Capability which conversation URI is equals to
	 * the gievn parameter
	 * 
	 * @param conversationURI
	 *            the URI of the conversation of the requested Capability
	 * @return the URI of the requested Capability, <i>null</i> if none
	 *         capability with such conversation URI has been found
	 */
	String getCapabilityWithConversationURI(String conversationURI) {
		for (Entry<String, Capability> entry : this.mapping.entrySet()) {
			if (entry.getValue().getConversationURI().equalsIgnoreCase(
					conversationURI))
				return entry.getValue().getCapabilityURI();
		}

		// if result not found
		return null;
	}

	/**
	 * This method create the Service from a file
	 * 
	 * @param uri :
	 *            the uri of this file
	 * @param getGrounding :
	 *            put the value to "true" if you want need to get the grounding,
	 *            "false" otherwise
	 * @return a complete Service Object
	 * @throws ParsingException
	 *             if an error occurs (see Exception message for more
	 *             information)
	 */
	public Service parseProcess(String uri, boolean getGrounding)
			throws ParsingException {

		String correctedURI = (Tools.seamsToBeAnURI(uri)) ? uri : Tools
				.getURIStringFromFilePath(uri);

		// initialise parsing
		this.initialiseParsing(correctedURI, getGrounding);

		// ask for parse the grounding
		this.parseGrounding();

		try {
			List<Individual> listCapa = this.getAllCapabilities();

			// DebugPrinter.print(listCapa,"list of capa",1);

			for (int i = 0; i < listCapa.size(); i++) {
				this.convertToSubCapability(listCapa.get(i));

			}

			List<Individual> listProfile = this.getAllServicesProfile();

			// DebugPrinter.print("Number Service Profil =
			// "+listProfile.size(),1);

			// Individual indi = listProfile.get(0);
			// DebugPrinter.print(indi.listProperties().toList(),"list of
			// property",1);

			try {
				Service s = this.convertToService(listProfile.get(0));

				// set QoS property
				s.setQos(this.getQoSProperty(correctedURI));

				// set the OWl file URI
				s
						.setOwlDescriptionFile(Tools
								.getURLStringFromURI(correctedURI));

				return s;
			} catch (ParsingException e) {
				DebugPrinter.print(e, DebugPrinter.WARN);
				throw e;
			}

		} catch (NullPointerException e) {
			DebugPrinter.print(e, DebugPrinter.WARN);
			throw new ParsingException("The URI " + uri
					+ " doesn't contain's any profile definition. (More :"
					+ e.getMessage() + ")");
		}
	}

	/**
	 * This method create the Service from a file
	 * 
	 * @param uri :
	 *            the uri of this file
	 * @param getGrounding :
	 *            put the value to "true" if you want need to get the grounding,
	 *            "false" otherwise
	 * @return a complete Service Object
	 * @throws ParsingException
	 *             if an error occurs (see Exception message for more
	 *             information)
	 * @throws IOException
	 *             if an error occurs with a file
	 */
	public Service parseProcessFromString(String Owl_sContent,
			boolean getGrounding) throws IOException, ParsingException {

		return this.parseProcess(Tools.createTempOwlFile(Owl_sContent),
				getGrounding);
	}

	/**
	 * Parse an Amigo-S description to generate a equivalent Service object. The
	 * wsdl URI given as second parameter will be associated to the description.
	 * The grounding part is also parsed.
	 * 
	 * @param semanticDescriptionURI
	 *            the URI of the file containing the amigo-S description
	 * 
	 * @param wsdlURI
	 *            the URi of the file containg the WSDL description
	 * @return a Service object equivalent to the Amigo-S description
	 * @throws ParsingException
	 *             if error occurs during parsing
	 */
	public Service parseProcess(String semanticDescriptionURI, String wsdlURI)
			throws ParsingException {
		Service result = this.parseProcess(semanticDescriptionURI, true);
		result.setWsdlDescriptionFile(wsdlURI);
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.parsers.ServiceParser#getImportsHashCode()
	 */
	@Override
	public String getImportsHashCode() {		
			return this.importsHashCode;		
	}

	/* (non-Javadoc)
	 * @see fr.inria.icocoa.servicemodel.parsers.ServiceParser#getListOfImports()
	 */
	@Override
	public List<String> getListOfImports() {
		return this.listOfImports;
	}
	
	

}
