/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel.parsers.jenaparser;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Map.Entry;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;

import fr.inria.icocoa.common.PropertyManager;
import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.ParameterBinding;
import fr.inria.icocoa.servicemodel.flow.CapabilityFlow;
import fr.inria.icocoa.servicemodel.flow.CompositeCapabilityFlow;
import fr.inria.icocoa.servicemodel.flow.CompositeCapabilityFlowFactory;
import fr.inria.icocoa.servicemodel.flow.FuturCapabilityFlow;
import fr.inria.icocoa.servicemodel.parsers.jenaparser.tools.ExtendedTools;
import fr.inria.icocoa.servicemodel.toolsobjects.CapabilityMap;
import fr.inria.icocoa.servicemodel.toolsobjects.Couple;
import fr.inria.icocoa.servicemodel.toolsobjects.ProcessElements;

/**
 * TaskParser.java <br>
 * WARNING : A capability can only have zero or one conversation. Each
 * Conversation is associated to only one capability
 * 
 * @author <a href="mailto:Galip-Ferda.Tartanoglu@inria.fr">Ferda Tartanoglu</a> -
 *         <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 */
public class TaskParser {

	/**
	 * The property manager used by this class
	 */
	private PropertyManager propertyManager = PropertyManager.getInstance();

	/**
	 * the current Ontology model
	 */
	private OntModel model;

	/**
	 * This map will contain the association between the CapabilityName and the
	 * corresponding conversation data = name of Conversation, key = name of the
	 * capability
	 */
	private TreeMap<String, String> capabilityProcess = new TreeMap<String, String>();

	/** Map of all binded_varaible (targetVariableName -> BindingObject) */
	private Map<String, ParameterBinding> binded_variable = new HashMap<String, ParameterBinding>();

	/** indicate if the map capabilityProcess has been already filled */
	private boolean mapAlreadyFilled = false;

	private String currentProcess = "";

	/** The parent parser */
	private SDL2ServiceParser parentParser;

	/**
	 * Constructor
	 * 
	 * @param model
	 *            the model of the ontology use for parsing.
	 * @param parser
	 *            the corresponding parser
	 * 
	 */
	public TaskParser(OntModel model, SDL2ServiceParser parser) {
		this.model = model;
		this.parentParser = parser;
	}

	/**
	 * Get the service name
	 * 
	 * @return the name of the service
	 */
	public String getServiceName() {

		String name = "";

		Resource serviceRes = model.getOntResource(this.propertyManager.__OWLS_SERVICE);

		// get first service
		Individual service = (Individual) model.listIndividuals(serviceRes).next();

		name = service.toString();
		return name;
	}

	/**
	 * Get the process and associated capability of the first component of a
	 * ControlConstructList
	 * 
	 * @return a Object containing many information about the process element
	 */
	@SuppressWarnings("unchecked")
	public CapabilityMap getFirst(Resource ccl) {

		// get first Process
		Statement st_first = ccl.getProperty(model.createProperty(this.propertyManager.__OBJLIST_FIRST));
		Resource node_first = (Resource) st_first.getObject();

		// DebugPrinter.print("node_first = "+node_first,1);
		// DebugPrinter.print(node_first.listProperties().toList(),"node_first.listProperties().toList()",1);

		// get first perform
		Statement st_first_process = node_first.getProperty(model.createProperty(this.propertyManager.__OWLS_PROCESS));
		Resource node_first_process = (Resource) st_first_process.getObject();

		// get first capability
		String capability = node_first_process.toString();

		Individual i = model.getIndividual(node_first_process.getURI());

		// DebugPrinter.print("Current process = "+node_first_process.getURI() ,
		// 1);

		CapabilityMap result = new CapabilityMap(capability,
				this.capabilityProcess.get(capability),
				i.getRDFType().toString());

		// test if there is some parameter binding
		Property hasDataFromType = model.createProperty(this.propertyManager.__OWLS_HAS_DATA_FROM);
		StmtIterator hasDataFromList = node_first.listProperties(hasDataFromType);
		if (hasDataFromList.hasNext()) {
			// DebugPrinter.print(node_first.listProperties().toList(),
			// "node_first.listProperties().toList()", 1);
			// DebugPrinter.print("current process = " + this.currentProcess,
			// 1);
			Map<String, ParameterBinding> bindings = this.parseParameterBindings(
				hasDataFromList, node_first_process.getURI());
			// DebugPrinter.print(bindings,"bindings",DebugPrinter.INFO);

			// store bindings in CapabilityMap
			for (Entry<String, ParameterBinding> entry : bindings.entrySet()) {
				if (entry.getValue().isInputBinding())
					result.getInputBinding().put(entry.getKey(),
						entry.getValue());
				else
					result.getOutputBinding().put(entry.getKey(),
						entry.getValue());
			}
		}

		// DebugPrinter.print("CapabilityMap = "+result, DebugPrinter.INFO);

		return result;
	}

	/**
	 * Store data binding in the map
	 * 
	 * @param parameterBindingList :
	 *            an Iterator on the list containing the declaration Parameter
	 *            binding
	 * @param currentCapability :
	 *            the name of the currentCapability that is parsed
	 */
	@SuppressWarnings("unchecked")
	private Map<String, ParameterBinding> parseParameterBindings(
			StmtIterator parameterBindingList, String currentCapability) {
		/*
		 * DebugPrinter.print("Parse data binding ProcessURI= " +
		 * this.currentProcess, DebugPrinter.INFO); DebugPrinter.print("Parse
		 * data binding currentCapability= " + currentCapability,
		 * DebugPrinter.INFO);
		 */

		Map<String, ParameterBinding> results = new HashMap<String, ParameterBinding>();

		// get the list of corresponding statement
		List<Statement> l = parameterBindingList.toList();

		// for each statement (each ParameterBinding)
		Property toParamType = model.createProperty(this.propertyManager.__OWLS_TO_PARAM);
		Property valueSourceType = model.createProperty(this.propertyManager.__OWLS_VALUE_SOURCE);
		Property theVarType = model.createProperty(this.propertyManager.__OWLS_THE_VAR);
		Property fromProcessType = model.createProperty(this.propertyManager.__OWLS_FROM_PROCESS);
		for (int i = 0; i < l.size(); i++) {
			// get the paramterBinding resource
			Resource parameterBindingResource = l.get(i).getResource();

			// get the type on parameterBinding
			boolean isInputType = this.testInputOutputBinding(parameterBindingResource);

			// get the ToValue property => the targetName
			String targetName = parameterBindingResource.getProperty(
				toParamType).asTriple().getObject().getURI();

			// get The groupe ValueSource
			Resource valueSourceResource = parameterBindingResource.getProperty(
				valueSourceType).getResource();

			// get the "TheVar" value => the sourceName
			String sourceName = valueSourceResource.getProperty(theVarType).asTriple().getObject().getURI();

			// get the "fromProcess" value => the sourceProcess
			String sourceProcess = valueSourceResource.getProperty(
				fromProcessType).asTriple().getObject().getURI();
			if (sourceProcess.equals(this.propertyManager.__OWLS_PARENT_PROCESS))
				sourceProcess = this.parentParser.getCapabilityWithConversationURI(this.currentProcess);
			// sourceProcess = ParameterBinding.THEPARENTPERFORM;
			/*
			 * sourceProcess = sourceProcess + PropertyManager.PROCESS_SEPARATOR +
			 * this.currentProcess;
			 */

			ParameterBinding toAdd = new ParameterBinding(targetName,
					currentCapability, sourceName, sourceProcess, isInputType);

			// add the parameterBinding to the map binded_variable
			this.binded_variable.put(targetName, toAdd);
			results.put(targetName, toAdd);
		}

		return results;
	}

	/**
	 * test if the ParameterType is an InputBinding or an OutputBinding
	 * 
	 * @param parameterBindingResource :
	 *            the resource corresponding to the ParamaterBinding
	 * @return true if it is a InputBinding, false Otherwise
	 */
	@SuppressWarnings("unchecked")
	private boolean testInputOutputBinding(Resource parameterBindingResource) {
		// get the rdf type to know if it is an InputBinding
		Property rdfTypeProperty = model.createProperty(this.propertyManager.__RDF_TYPE);
		StmtIterator typeList = parameterBindingResource.listProperties(rdfTypeProperty);
		boolean isInputType = false;
		List<Statement> listOfType = typeList.toList();
		for (int j = 0; (j < listOfType.size() && !isInputType); j++) {
			try {
				Statement type = listOfType.get(j);
				String s = type.asTriple().getObject().getURI();
				isInputType = s.equals(this.propertyManager.__OWLS_INPUT_BINDING);
			} catch (UnsupportedOperationException e) {
				// for some given statement, the Object is not an URI, so the
				// operation getURI return UnsupportedOperationException
				// this means that we didn't care this type and we ignore it.
				isInputType = false;
			}
		}
		return isInputType;
	}

	/**
	 * Get the Process control construct + elements
	 * 
	 * @return a list of all the elements of the process.
	 */
	private ProcessElements getProcessElements(String process_uri) {

		// initialize the result
		ProcessElements pe = new ProcessElements();

		// get the part of description that describe the process
		Resource process = model.getOntResource(process_uri);
		Property composedOf = model.createProperty(this.propertyManager.__OWLS_COMPOSED_OF);
		Statement st = process.getProperty(composedOf);
		Resource node = (Resource) st.getObject();
		Property rdfType = model.createProperty(this.propertyManager.__RDF_TYPE);
		Statement st_ = node.getProperty(rdfType);

		// get the name of the ConstrolConstruct
		Resource cons = (Resource) st_.getObject();
		pe.setControl(cons.getLocalName());

		/* *** get first and rests *** */

		// get control construct list
		Statement st_comp = node.getProperty(model.createProperty(this.propertyManager.__OWLS_COMPONENTS));
		Resource node_comp = (Resource) st_comp.getObject();

		// get first conversation
		CapabilityMap cm = getFirst(node_comp);

		if (cm != null)
			pe.addElement(cm);

		// and the rest..

		boolean done = false;

		Resource node_rest;
		Statement st_rest;

		while (!done) {

			st_rest = node_comp.getProperty(model.createProperty(this.propertyManager.__OBJLIST_REST));
			node_rest = (Resource) st_rest.getObject();

			if (!node_rest.toString().equals(this.propertyManager.__OBJLIST_NIL)) {

				cm = this.getFirst(node_rest);
				pe.addElement(cm);

				node_comp = node_rest;
			} else {
				done = true;
			}
		}

		return pe;
	}

	/**
	 * Get the Flow describe as a Capability's conversation
	 * 
	 * @param process_uri
	 *            the name of the conversation
	 * @return the described flow
	 * @throws CloneNotSupportedException
	 *             if an error occurs during cloning of ParameterBinding or Flow
	 * @throws ParsingException
	 *             if the CapabilityMap object contains an unknown URI as
	 *             capability name
	 */
	private CapabilityFlow getFlow(String process_uri)
			throws CloneNotSupportedException, ParsingException {

		// get the part of description that describe the process
		Resource process = model.getOntResource(process_uri);
		Property composedOf = model.createProperty(this.propertyManager.__OWLS_COMPOSED_OF);
		Statement st = process.getProperty(composedOf);
		Resource node = (Resource) st.getObject();
		Property rdfType = model.createProperty(this.propertyManager.__RDF_TYPE);
		Statement st_ = node.getProperty(rdfType);

		// get the name of the ConstrolConstruct
		Resource cons = (Resource) st_.getObject();
		String controlConstructName = cons.getLocalName();

		// get the ControlConstruct flow container
		CompositeCapabilityFlow result = CompositeCapabilityFlowFactory.getInstance(
			controlConstructName, new Object[] {});

		// check if not null
		if (result == null)
			return null;

		/*
		 * get children
		 */

		// get control construct list
		Statement st_comp = node.getProperty(model.createProperty(this.propertyManager.__OWLS_COMPONENTS));
		Resource node_comp = (Resource) st_comp.getObject();

		// get first conversation
		CapabilityMap cm = this.getFirst(node_comp);

		if (cm != null) {
			this.treatFirst(cm, result);
		}

		// and the rest..

		boolean done = false;

		Resource node_rest;
		Statement st_rest;

		while (!done) {

			st_rest = node_comp.getProperty(model.createProperty(this.propertyManager.__OBJLIST_REST));
			node_rest = (Resource) st_rest.getObject();

			if (!node_rest.toString().equals(this.propertyManager.__OBJLIST_NIL)) {

				cm = this.getFirst(node_rest);
				if (cm != null) {
					this.treatFirst(cm, result);
				}

				node_comp = node_rest;
			} else {
				done = true;
			}
		}

		return result;
	}

	/**
	 * Get the Flow describe as a Capability's conversation and all the process
	 * elements that are used to perform this flow
	 * 
	 * @param process_uri
	 *            the name of the conversation
	 * @return a couple ProcessElements/CapabilityFlow corresponding to the
	 *         conversation, <i>null</i> of an error has been thrown during
	 *         parsing.
	 * @throws CloneNotSupportedException
	 *             if an error occurs during cloning of ParameterBinding or Flow
	 * @throws ParsingException
	 *             if the CapabilityMap object contains an unknown URI as
	 *             capability name
	 */
	public Couple<ProcessElements, CapabilityFlow> getFlowAndProcess(
			String process_uri) throws CloneNotSupportedException,
			ParsingException {

		assert (process_uri != null);

		// DebugPrinter.print("process_uri = " + process_uri,
		// DebugPrinter.INFO);

		// update the URI of the process it parse
		this.currentProcess = process_uri;

		// create the map processName/capabilityName
		this.fillTheMapOfConversation();

		// get the process elements
		ProcessElements pe = this.getProcessElements(process_uri);
		if (pe == null)
			return null;

		// get the capability flow
		CapabilityFlow cf = this.getFlow(process_uri);

		if (cf == null)
			return null;

		// return the result
		return new Couple<ProcessElements, CapabilityFlow>(pe, cf);

	}

	/**
	 * This method complete the Map containing the name of the Capability
	 * associated with each process
	 * 
	 */
	@SuppressWarnings("unchecked")
	private void fillTheMapOfConversation() {

		// check if the map has been already filled
		if (!this.mapAlreadyFilled) {

			// create resource object that will be used during parsing
			Resource capaRes = model.getOntResource(this.propertyManager.__SERVICE_CAPABILITY);
			Property hasProcessProp = model.createProperty(this.propertyManager.__HASCONVERSATION_PROPERTY);

			// list all capabilities
			List<Individual> lcapa = model.listIndividuals(capaRes).toList();

			// for each capability store the association
			// processName/capabilityName
			for (int i = 0; i < lcapa.size(); i++) {
				Individual indi = lcapa.get(i);

				Statement conversationSt = indi.getProperty(hasProcessProp);
				Individual hasConversationIndi = ExtendedTools.getIndividualFromStatement(
					model, conversationSt);
				this.capabilityProcess.put(indi.getURI(),
					hasConversationIndi.getURI());
			}

			// update the variable to avoid multiple filling of the map
			this.mapAlreadyFilled = true;
		}
	}

	/**
	 * @return the binded_variable
	 */
	public Map<String, ParameterBinding> getBinded_variable() {
		return binded_variable;
	}

	/**
	 * @param binded_variable
	 *            the binded_variable to set
	 */
	public void setBinded_variable(Map<String, ParameterBinding> binded_variable) {
		this.binded_variable = binded_variable;
	}

	/*
	 * Change the binded variable "TheParentProcess" into the name od capability
	 * corresponding to the process
	 * 
	 * @param binded_variable : the map iof binded variable to check and
	 * transform @param s : the service object from which binded variables are
	 * extract
	 */
	/*
	 * public static void solveTheParentPerform( Map<String, ParameterBinding>
	 * binded_variable, Service s) { //
	 * DebugPrinter.print(binded_variable,"binded_variable", 1); }
	 */

	/**
	 * Copy the required value from CapabilityMap to CapabilityMap
	 * 
	 * @param cm
	 *            the CapabilityMap containing the information to copy
	 * @param result
	 *            the Flow that required those informations
	 * @throws CloneNotSupportedException
	 *             if an error occurs during cloning of ParameterBinding or Flow
	 * @throws ParsingException
	 *             if the CapabilityMap object contains an unknown URI as
	 *             capability name
	 */
	private void treatFirst(CapabilityMap cm, CompositeCapabilityFlow result)
			throws CloneNotSupportedException, ParsingException {

		// get the capability that will be invoke !
		Capability capa = this.parentParser.getMapping().get(cm.getCapability());

		if (capa != null) {
			CapabilityFlow clone = null;
			if (capa.getFlow() != null) {
				clone = capa.getFlow().clone();

				if (clone instanceof FuturCapabilityFlow) {
					((FuturCapabilityFlow) clone).addNewReferenceOnI(result);
				}

				result.addChild(clone);
			} else {
				clone = new FuturCapabilityFlow(cm.getCapability(), result);
				result.addChild(clone);
			}

			/* Treat inputs binding */
			Map<String, ParameterBinding> inputs = cm.getInputBinding();
			for (Entry<String, ParameterBinding> entry : inputs.entrySet())
				clone.addInputsBinding(entry.getKey(), entry.getValue());

			/* Treat outputs binding */
			Map<String, ParameterBinding> outputs = cm.getOutputBinding();
			for (Entry<String, ParameterBinding> entry : outputs.entrySet())
				clone.addOutputsBinding(entry.getKey(), entry.getValue());

		} else {
			throw new ParsingException("the Capability with URI "
					+ cm.getCapability() + " is unknown");
		}

	}
}
