/**
 * The contents of this file are subject to the BT "ZEUS" Open Source 
 * Licence (L77741), Version 1.0 (the "Licence"); you may not use this file 
 * except in compliance with the Licence. You may obtain a copy of the Licence
 * from $ZEUS_INSTALL/licence.html or alternatively from
 * http://www.labs.bt.com/projects/agents/zeus/licence.htm
 * 
 * Except as stated in Clause 7 of the Licence, software distributed under the
 * Licence is distributed WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the Licence for the specific language governing rights and 
 * limitations under the Licence.
 * 
 * The Original Code is within the package zeus.*.
 * The Initial Developer of the Original Code is British Telecommunications
 * public limited company, whose registered office is at 81 Newgate Street, 
 * London, EC1A 7AJ, England. Portions created by British Telecommunications 
 * public limited company are Copyright 1996-9. All Rights Reserved.
 * 
 * THIS NOTICE MUST BE INCLUDED ON ANY COPY OF THIS FILE
 */

package zeus.util;

import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.util.Vector;

/**
 * 
 * TreeNode.java<br>
 * This class under GPL v1, has been modified and commented by <a
 * href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 * href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a> to be
 * compliant with Java 1.5.<br>
 * <br>
 * 
 * This class implements a node of a tree, and the different algorithm to deal
 * with tree.
 * 
 * 
 * Creation Date : Jul 18, 2007 5:03:41 PM
 * 
 * @param <T>
 *            the type of data that will be contained in the Tree
 */
public class TreeNode<T> {
	/** indicate that the enumartor works on Node */
	protected static final int NODES = 0;

	/** indicate that the enumartor works on value */
	protected static final int VALUES = 1;

	/** The object associate with the node */
	protected T data = null;

	/** The list of child of the node */
	protected Vector<TreeNode<T>> children = null;

	/** The parent node */
	protected TreeNode<T> parent = null;

	/**
	 * Create a new node by specifiing the parent node and the associated object
	 * 
	 * @param parent
	 *            the parent node
	 * @param data
	 *            the associated object
	 */
	public TreeNode(TreeNode<T> parent, T data) {
		children = new Vector<TreeNode<T>>();
		setParent(parent);
		setValue(data);
	}

	/**
	 * Create a new node by specifiing the associated object. The parent node is
	 * set to <i>null</i>
	 * 
	 * @param data
	 *            the associated object
	 */
	public TreeNode(T data) {
		children = new Vector<TreeNode<T>>();
		setValue(data);
	}

	/**
	 * 
	 * TreeNode.java<br>
	 * This class under GPL v1, has been modified and commented by <a
	 * href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
	 * href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a> to be
	 * compliant with Java 1.5.<br>
	 * <br>
	 * 
	 * This class implement a Enumerator for our tree.
	 * 
	 * Modification Date : Jul 18, 2007 5:21:17 PM
	 */
	class PreorderEnumerator implements Enumeration {

		/** the current node */
		TreeNode<T> current = null;

		/** The root node of the algorithm */
		TreeNode<T> root = null;

		/**
		 * the type of value to enumerate (TreeNode.NODE = <TreeNode<T>>,
		 * TreeNode.VALUES = T)
		 */
		int type;

		/**
		 * Constructor of the Enumerator
		 * 
		 * @param node
		 *            the root node for algorithm
		 * @param type
		 *            the type of value to enumerate (TreeNode.NODE = <TreeNode<T>>,
		 *            TreeNode.VALUES = T)
		 */
		public PreorderEnumerator(TreeNode<T> node, int type) {
			root = current = node;
			this.type = type;
		}

		public boolean hasMoreElements() {
			return current != null;
		}

		public Object nextElement() {
			if (current == null)
				throw new NoSuchElementException();

			TreeNode<T> token = current;

			current = token.firstChild();
			if (current == null && token != root)
				current = token.nextSibling();
			if (current == null) {
				TreeNode<T> parent = (token == root) ? null : token.getParent();
				while (parent != null && parent != root
						&& (current = parent.nextSibling()) == null)
					parent = (parent == root) ? null : parent.getParent();
			}

			if (type == NODES)
				return token;
			else if (token != null)
				return token.getValue();
			else
				return null;
		}
	}

	/**
	 * Get an enumeration of objects contained in tree
	 * 
	 * @return an Enumeration of <T> object
	 */
	@SuppressWarnings("unchecked")
	public Enumeration<T> values() {
		return new PreorderEnumerator(this, VALUES);
	}

	/**
	 * Get an enumeration of the nodes of the tree
	 * 
	 * @return an Enumeration of <TreeNode<T>> object
	 */
	@SuppressWarnings("unchecked")
	public Enumeration<TreeNode<T>> nodes() {
		return new PreorderEnumerator(this, NODES);
	}

	/**
	 * Test if the current node is the root of the tree or not
	 * 
	 * @return <i>true</i> if this node if the root of the tree, <i>false</i>
	 *         otherwise
	 */
	public boolean isRoot() {
		return parent == null;
	}

	/**
	 * Test if the current node is a leaf of the tree or not
	 * 
	 * @return <i>true</i> if this node if a leaf of the tree, <i>false</i>
	 *         otherwise
	 */
	public boolean isTerminal() {
		return children.isEmpty();
	}

	/**
	 * Test if the current node as children
	 * 
	 * @return <i>true</i> if this node has children, <i>false</i> otherwise
	 */
	public boolean hasChildren() {
		return !children.isEmpty();
	}

	/**
	 * Get the parent node of the current one
	 * 
	 * @return the parent node of the current one
	 */
	public TreeNode<T> getParent() {
		return parent;
	}

	/**
	 * Get the associated object of the node
	 * 
	 * @return the associated object of the node
	 */
	public T getValue() {
		return data;
	}

	/**
	 * Associate a new object with the current node. (the old one will be
	 * overwrite)
	 * 
	 * @param value :
	 *            the new object to associate with this node
	 */
	public void setValue(T value) {
		Assert.notNull(value);
		data = value;
	}

	/**
	 * Get the list of children of the node
	 * 
	 * @return the list of children of the current node
	 */
	public Vector<TreeNode<T>> getChildren() {
		Vector<TreeNode<T>> result = new Vector<TreeNode<T>>();
		for (int i = 0; i < children.size(); i++)
			result.addElement(children.elementAt(i));
		return result;
	}

	/**
	 * test if a node is contains in the list of children of the current node
	 * 
	 * @param child
	 *            the searched node
	 * @return <i>true</i> if the node is already contains in the list of
	 *         children, <i>false</i> otherwise
	 */
	public boolean hasChild(TreeNode<T> child) {
		return children.contains(child);
	}

	/**
	 * Set the parent node of the current one
	 * 
	 * @param parent :
	 *            the parent node
	 */
	public void setParent(TreeNode<T> parent) {
		this.parent = parent;
	}

	/**
	 * Test if a value is contained in the branch whose root is the current node
	 * 
	 * @param data
	 *            we want to locate
	 * @return <i>true</i> if the value is found,<i>false</i> otherwise
	 */
	public boolean containsValue(T data) {
		if (this.data.equals(data))
			return true;
		boolean status = false;
		TreeNode<T> node;
		for (int i = 0; !status && i < children.size(); i++) {
			node = children.elementAt(i);
			status |= node.containsValue(data);
		}
		return status;
	}

	/**
	 * Add a child to the current node (this node will be add at the end of the
	 * list of children)
	 * 
	 * @param node
	 *            the node to add
	 * @return the node that as just been added
	 */
	public TreeNode<T> addChild(TreeNode<T> node) {
		node.setParent(this);
		children.addElement(node);
		return node;
	}

	/**
	 * Add a node with associated data (this node will be create) in the list of
	 * child of the current node (this node will be add at the end of the list
	 * of children)
	 * 
	 * @param data
	 *            the value to add in the tree
	 * @return the node that as just been added
	 */
	public TreeNode<T> addChild(T data) {
		TreeNode<T> node = new TreeNode<T>(this, data);
		children.addElement(node);
		return node;
	}

	/**
	 * Add a child to the current node (this node will be add at the given
	 * position of the list of children)
	 * 
	 * @param node
	 *            the node to add
	 * @param position
	 *            the position to add the new child
	 * @return the node that as just been added
	 */
	public TreeNode<T> addChild(TreeNode<T> node, int position) {
		Assert.notFalse(position >= 0 && position <= children.size());
		node.setParent(this);
		children.insertElementAt(node, position);
		return node;
	}

	/**
	 * Add a node with associated data (this node will be create) in the list of
	 * child of the current node (this node will be add at the given position of
	 * the list of children)
	 * 
	 * @param data
	 *            the value to add in the tree
	 * @param position
	 *            the position to add the new child
	 * @return the node that as just been added
	 */
	public TreeNode<T> addChild(T data, int position) {
		Assert.notFalse(position >= 0 && position <= children.size());
		TreeNode<T> node = new TreeNode<T>(this, data);
		children.insertElementAt(node, position);
		return node;
	}

	/**
	 * Remove a child from the list of children
	 * 
	 * @param node
	 *            the node to remove
	 */
	public void removeChild(TreeNode<T> node) {
		children.removeElement(node);
	}

	/**
	 * Remove a child from the list of children
	 * 
	 * @param position
	 *            the position in the list of child of the removed node
	 */
	public void removeChild(int position) {
		Assert.notFalse(position >= 0 && position < children.size());
		children.removeElementAt(position);
	}

	/**
	 * Add a given Data in the list of children, just before a given node
	 * (create the associated node)
	 * 
	 * @param data
	 *            the data to insert
	 * @return the node that as just been added
	 */
	public TreeNode<T> addBefore(T data) {
		Assert.notNull(parent);
		TreeNode<T> node = new TreeNode<T>(parent, data);
		parent.addBefore(node, this);
		return node;
	}

	/**
	 * Add an node in the list of children just before a given one
	 * 
	 * @param node
	 *            the node to insert
	 * @return the node that as just been added
	 */
	public TreeNode<T> addBefore(TreeNode<T> node) {
		Assert.notNull(parent);
		node.setParent(parent);
		parent.addBefore(node, this);
		return node;
	}

	/**
	 * Add a given Data in the list of children, just after a given node (create
	 * the associated node)
	 * 
	 * @param data
	 *            the data to insert
	 * @return the node that as just been added
	 */
	public TreeNode<T> addAfter(T data) {
		Assert.notNull(parent);
		TreeNode<T> node = new TreeNode<T>(parent, data);
		parent.addAfter(node, this);
		return node;
	}

	/**
	 * Add an node in the list of children just after a given one
	 * 
	 * @param node
	 *            the node to insert
	 * @return the node that as just been added
	 */
	public TreeNode<T> addAfter(TreeNode<T> node) {
		Assert.notNull(parent);
		node.setParent(parent);
		parent.addAfter(node, this);
		return node;
	}

	/**
	 * Submethod that permit to add a node just before another one recursivly
	 * 
	 * @param node
	 *            the node to insert
	 * @param me
	 *            the current node
	 */
	protected void addBefore(TreeNode<T> node, TreeNode<T> me) {
		for (int i = 0; i < children.size(); i++) {
			if (children.elementAt(i).equals(me)) {
				children.insertElementAt(node, i);
				return;
			}
		}
		Assert.notNull(null);
	}

	/**
	 * Submethod that permit to add a node just after another one recursivly
	 * 
	 * @param node
	 *            the node to insert
	 * @param me
	 *            the current node
	 */
	protected void addAfter(TreeNode<T> node, TreeNode<T> me) {
		for (int i = 0; i < children.size(); i++) {
			if (children.elementAt(i).equals(me)) {
				children.insertElementAt(node, i + 1);
				return;
			}
		}
		Assert.notNull(null);
	}

	/**
	 * Get the next brother of the current node
	 * 
	 * @return a brother of the current node
	 */
	public TreeNode<T> nextSibling() {
		return parent != null ? parent.nextSibling(this) : null;
	}

	/**
	 * Get the previous brother of the current node
	 * 
	 * @return a brother of the current node
	 */
	public TreeNode<T> previousSibling() {
		return parent != null ? parent.previousSibling(this) : null;
	}

	/**
	 * Submethod that permit ot obtain the next brother of a node
	 * 
	 * @param me
	 *            the current node
	 * @return the node just after the current one in the list of child of the
	 *         parent of "me"
	 */
	protected TreeNode<T> nextSibling(TreeNode<T> me) {
		for (int i = 0; i < children.size(); i++) {
			if (children.elementAt(i).equals(me)) {
				if (i + 1 < children.size())
					return children.elementAt(i + 1);
				else
					return null;
			}
		}
		Assert.notNull(null); // I am not in my dad's list of children
		return null;
	}

	/**
	 * Submethod that permit ot obtain the previous brother of a node
	 * 
	 * @param me
	 *            the current node
	 * @return the node just after the current one in the list of child of the
	 *         parent of "me"
	 */
	protected TreeNode<T> previousSibling(TreeNode<T> me) {
		for (int i = 0; i < children.size(); i++) {
			if (children.elementAt(i).equals(me)) {
				if (i - 1 >= 0)
					return children.elementAt(i - 1);
				else
					return null;
			}
		}
		Assert.notNull(null); // I am not in my dad's list of children
		return null;
	}

	/**
	 * Get the first child (node) of the current node
	 * 
	 * @return a node of the Tree.
	 */
	public TreeNode<T> firstChild() {
		return children.isEmpty() ? null : children.elementAt(0);
	}

	/**
	 * Get a string representation of the object attached to the node (use the
	 * toString() method of this object
	 * 
	 * @return a string representation of the object attached to the node
	 */
	@Override
	public String toString() {
		return data.toString();
	}

	/**
	 * This method permit to obtain a String description of an under-tree <br>
	 * #Added by Sebastien Bianco#<br>
	 * 
	 * @param prefix :
	 *            the prefix to put before each line to obtain a good
	 *            indentation
	 * @return the String corresponding to the description of the under-tree
	 */
	public String recursiveToString(String prefix) {
		if (this.isTerminal())
			return prefix + "- " + data.toString() + "\n";
		else {
			String result = prefix + "+ " + data.toString() + "\n";
			for (int i = 0; i < children.size(); i++) {
				result += children.get(i).recursiveToString(prefix + "\t");
			}
			return result;
		}
	}

	/**
	 * Get the size of the branch whose root is the current node<br>
	 * #Added by Sebastien Bianco#<br>
	 * 
	 * @return the size of the branch (at least = 1, if the node is a terminal
	 *         node)
	 */
	public int size() {
		if (this.isTerminal())
			return 1;
		else {
			int result = 1;
			for (int i = 0; i < children.size(); i++) {
				result += children.get(i).size();
			}
			return result;
		}
	}

}
