/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.junittest.servicemodel.parsers;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.junit.Test;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.ControlConstruct;
import fr.inria.icocoa.servicemodel.GroundingProperty;
import fr.inria.icocoa.servicemodel.Output;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.parsers.ServiceParserFactory;
import fr.inria.icocoa.servicemodel.parsers.jenaparser.SDL2ServiceParser;

/**
 * SDL2ServiceParserTest.java
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : May 14, 2007 2:59:03 PM
 */
public class SDL2ServiceParserTest {

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.semanticcomposition.sdlparser.SDL2ServiceParser#parseProcessFromString(java.lang.String, boolean)}.
	 */
	@Test
	public final void testParseProcessFromStringVerySimpleService() {

		String SDLfilename = System.getProperty("user.dir")
				+ "/amigo-s/VerySimpleService.owl";
		String service = Tools.readFromFile(SDLfilename);

		try {
			if ((new File(SDLfilename)).exists()) {

				// //////////////////////////
				// Test Without Grounding //
				// //////////////////////////

				/* get service from parsing */
				Long time = System.currentTimeMillis();
				Service s = ServiceParserFactory.getInstance().getParser().parseProcessFromString(service,
						false);
				time = System.currentTimeMillis() - time;
				DebugPrinter.print("\nparsing " + SDLfilename
						+ "time without Grounding = " + time + " ms", 1);

				/* Construct equivalent service */
				Output out = new Output(
						"Op1OutputSemantic",
						"http://amigo.gforge.inria.fr/owl/Capabilities.owl",
						"http://amigo.gforge.inria.fr/owl/VerySimpleService.owl#ServiceAOp1Output");
				out.setProvided(true);

				Capability c = new Capability();
				c
						.setConversationURI("http://amigo.gforge.inria.fr/owl/VerySimpleService.owl#ServiceAOp1Conversation");
				c.setAtomic(true);
				c
						.setCapabilityURI("http://amigo.gforge.inria.fr/owl/VerySimpleService.owl#ServiceAOp1Capability");
				c
						.setType("http://amigo.gforge.inria.fr/owl/Capabilities.owl#Op1Semantic");
				c.setLocalName("ServiceAOp1Capability");
				c.addOutput(out);

				Service s2 = new Service();
				s2.addProvidedCapability(c);
				s2
						.setBase("http://amigo.gforge.inria.fr/owl/VerySimpleService.owl");
				s2.setLocalName("VerySimpleService");
				s2.setType("VerySimpleServiceType");
				c.setService(s2);

				// hack to fix description file (if not, equals will return
				// false)
				s2.setOwlDescriptionFile(s.getOwlDescriptionFile());

				/* test */

				// test not null
				assertNotNull("test service not null", s);
								
				// test equals s1 s2
				assertEquals("test of equality", s, s2);

				// ///////////////////////
				// Test With Grounding //
				// ///////////////////////

				/* get service from parsing */
				time = System.currentTimeMillis();
				Service s3 = ServiceParserFactory.getInstance().getParser().parseProcessFromString(service,
						true);
				time = System.currentTimeMillis() - time;
				DebugPrinter.print("\nparsing " + SDLfilename
						+ "time with grounding = " + time + " ms", 1);

				/* Complete equivalent service */
				out
						.setGroundingValue("http://amigo.gforge.inria.fr/owl/VerySimpleService.wsdl#op1Return");

				GroundingProperty gp = new GroundingProperty();
				gp
						.setConversationURI("http://amigo.gforge.inria.fr/owl/VerySimpleService.owl#ServiceAOp1Conversation");
				gp
						.setOperationName("http://amigo.gforge.inria.fr/owl/VerySimpleService.wsdl#op1");
				gp
						.setOperationPortType("http://amigo.gforge.inria.fr/owl/VerySimpleService.wsdl#ServiceA");
				gp
						.setWsdlFileName("http://amigo.gforge.inria.fr/owl/VerySimpleService.wsdl");
				gp
						.setWsdlInputMessage("http://amigo.gforge.inria.fr/owl/VerySimpleService.wsdl#op1Request");
				gp
						.setWsdlOutputMessage("http://amigo.gforge.inria.fr/owl/VerySimpleService.wsdl#op1Response");

				c.setGrounding(gp);

				// hack to fix description file (if not, equals will return
				// false)
				s2.setOwlDescriptionFile(s3.getOwlDescriptionFile());

				/* test */

				// test not null
				assertNotNull("test service not null (grounding)", s3);

				// test equals s1 s2
				assertEquals("test of equality of grounding property", gp, s3
						.getProvidedCapabilities().get(0).getGrounding());
				assertTrue("test of equality without Capability", s3
						.equalsWithoutCapabilities(s2));
				assertEquals("test of equality (grounding)", s3, s2);

			} else {
				fail("file " + SDLfilename
						+ " not found ! Test cannot be perform");
			}

		} catch (ParsingException e) {
			fail("Parsing Exception (" + e.getMessage() + ")");
		} catch (IOException e) {
			fail("IO Exception (" + e.getMessage() + ")");
		}

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.semanticcomposition.sdlparser.SDL2ServiceParser#parseProcess(java.lang.String, boolean)}.
	 */
	@Test
	public final void testParseProcessMultimediaBrowerService() {
		String SDLfilename = System.getProperty("user.dir")
				+ "/amigo-s/MultimediaBrowser.owl";

		try {
			if ((new File(SDLfilename)).exists()) {

				// //////////////////////////
				// Test Without Grounding //
				// //////////////////////////

				/* get service from parsing */
				Long time = System.currentTimeMillis();
				Service s = ServiceParserFactory.getInstance().getParser().parseProcess(Tools
						.getURIStringFromFilePath(SDLfilename), false);
				time = System.currentTimeMillis() - time;
				DebugPrinter.print("\nparsing " + SDLfilename
						+ "time without grounding = " + time + " ms", 1);

				/* Construct equivalent service */
				Output out = new Output("BrowseContent",
						"http://www.dyonisos.org/functionalities.owl",
						"http://www.dyonisos.org/MultimediaBrowser.owl#BrowseContentOutput");
				out.setProvided(false);

				Output outp = new Output("BrowseContent",
						"http://www.dyonisos.org/functionalities.owl",
						"http://www.dyonisos.org/MultimediaBrowser.owl#BrowseContentOutput");
				outp.setProvided(true);

				Output out2 = new Output("Information",
						"http://www.dyonisos.org/functionalities.owl",
						"http://www.dyonisos.org/MultimediaBrowser.owl#SetOnOutput");
				out2.setProvided(false);

				Output out2p = new Output("Information",
						"http://www.dyonisos.org/functionalities.owl",
						"http://www.dyonisos.org/MultimediaBrowser.owl#SetOnOutput");
				out2p.setProvided(true);

				Capability c2 = new Capability();
				c2
						.setConversationURI("http://www.dyonisos.org/MultimediaBrowser.owl#SetOnConversation");
				c2.setAtomic(true);
				c2
						.setCapabilityURI("http://www.dyonisos.org/MultimediaBrowser.owl#SetOn");
				c2
						.setType("http://amigo.gforge.inria.fr/owl/Capabilities.owl#SetOn");
				c2.setLocalName("SetOn");
				c2.addOutput(out2);

				Capability c3 = new Capability();
				c3
						.setConversationURI("http://www.dyonisos.org/MultimediaBrowser.owl#BrowseContentConversation");
				c3.setAtomic(true);
				c3
						.setCapabilityURI("http://www.dyonisos.org/MultimediaBrowser.owl#BrowseContent");
				c3
						.setType("http://amigo.gforge.inria.fr/owl/Capabilities.owl#BrowseContent");
				c3.setLocalName("BrowseContent");
				c3.addOutput(out);

				Capability c = new Capability();
				c
						.setConversationURI("http://www.dyonisos.org/MultimediaBrowser.owl#MMBrowserConversation");
				c.setAtomic(false);
				c
						.setCapabilityURI("http://www.dyonisos.org/MultimediaBrowser.owl#MMBrowserCapability");
				c
						.setType("http://amigo.gforge.inria.fr/owl/Capabilities.owl#MediaServer");
				c.setLocalName("MMBrowserCapability");
				c.addOutput(outp);
				c.addOutput(out2p);
				c.setCC(ControlConstruct.SEQUENCE);
				c.addSubCapability(c2);
				c.addSubCapability(c3);

				Service s2 = new Service();
				s2.addProvidedCapability(c);
				s2.addCapability(c2);
				s2.addCapability(c3);
				s2.setBase("http://www.dyonisos.org/MultimediaBrowser.owl");
				s2.setLocalName("MultimediaBrowserService");
				s2
						.setType("http://amigo.gforge.inria.fr/owl/Capabilities.owl#MultimediaBrowser");

				s2.setOwlDescriptionFile(Tools
						.getURIStringFromFilePath(SDLfilename));
				c.setService(s2);
				c2.setService(s2);
				c3.setService(s2);
				
				

				/* test */
				//s.print(System.out);
				//s2.print(System.out);
				
				assertEquals("test output BrowseContent",out,
						(new ArrayList<Output>(s.getCapabilityFromTypeName("BrowseContent").get(0)
								.getOutputs().values()).get(0)));
				assertEquals("test output SetOn",
						(new ArrayList<Output>(s.getCapabilityFromTypeName("SetOn").get(0)
								.getOutputs().values()).get(0)), out2);
				assertEquals("test capa 1", s.getCapabilityFromTypeName("BrowseContent").get(0), c3);
				assertEquals("test capa 2", s.getCapabilityFromTypeName("SetOn").get(0), c2);
				/*assertEquals("test output BrowseContent 2",
						(new ArrayList<Output>(s.getProvidedCapabilities().get(
								0).getOutputs().values()).get(1)), out2p);
				assertEquals("test output SetOn 2 ",
						(new ArrayList<Output>(s.getProvidedCapabilities().get(
								0).getOutputs().values()).get(0)), outp);*/
				assertEquals("test capa 3", s.getProvidedCapabilities().get(0)
						.getSubCapabilities().get(0), c2);
				assertEquals("test capa 4", s.getProvidedCapabilities().get(0)
						.getSubCapabilities().get(1), c3);

				Capability provided = s.getProvidedCapabilities().get(0);
				assertEquals("same type", provided.getType(), c.getType());
				assertEquals("same localName", provided.getLocalName(), c
						.getLocalName());
				assertEquals("same URI", provided.getCapabilityURI(), c
						.getCapabilityURI());
				assertEquals("same ConversationURI", provided
						.getConversationURI(), c.getConversationURI());
				assertEquals("same atomicity", provided.isAtomic(), c
						.isAtomic());
				assertEquals("same inputs", provided.getInputs(), c.getInputs());
				assertEquals("same outputs", provided.getOutputs(), c
						.getOutputs());
				assertEquals("same subcapa", provided.getSubCapabilities(), c
						.getSubCapabilities());
				assertTrue("test equals Capa withoutSubcpabilities", provided
						.equalsWithoutSubCapabilities(c));

				assertEquals("test provided capa", provided, c);

				// s.print(System.out);
				// s2.print(System.out);

				assertTrue("test equals Service without SubCapabilities", s
						.equalsWithoutCapabilities(s2));

				// test not null
				assertNotNull("test service not null", s);
				// test equals s1 s2
				assertEquals("test of equality", s, s2);

				// ///////////////////////
				// Test With Grounding //
				// ///////////////////////

				/* get service from parsing */
				time = System.currentTimeMillis();
				Service s3 = ServiceParserFactory.getInstance().getParser().parseProcess(Tools
						.getURIStringFromFilePath(SDLfilename), true);
				time = System.currentTimeMillis() - time;
				DebugPrinter.print("\nparsing " + SDLfilename
						+ "time with grounding = " + time + " ms", 1);

				/* Complete equivalent service */
				out
						.setGroundingValue("http://www.dyonisos.org/MultimediaBrowser.wsdl#browseReturn");
				outp
						.setGroundingValue("http://www.dyonisos.org/MultimediaBrowser.wsdl#browseReturn");
				out2
						.setGroundingValue("http://www.dyonisos.org/MultimediaBrowser.wsdl#setOnReturn");
				out2p
						.setGroundingValue("http://www.dyonisos.org/MultimediaBrowser.wsdl#setOnReturn");

				GroundingProperty gp = new GroundingProperty();
				gp
						.setConversationURI("http://www.dyonisos.org/MultimediaBrowser.owl#BrowseContentConversation");
				gp
						.setOperationName("http://www.dyonisos.org/MultimediaBrowser.wsdl#browse");
				gp
						.setOperationPortType("http://www.dyonisos.org/MultimediaBrowser.wsdl#MultimediaBrowser");
				gp
						.setWsdlFileName("http://www.dyonisos.org/MultimediaBrowser.wsdl");
				gp
						.setWsdlInputMessage("http://www.dyonisos.org/MultimediaBrowser.wsdl#browseRequest");
				gp
						.setWsdlOutputMessage("http://www.dyonisos.org/MultimediaBrowser.wsdl#browseResponse");

				GroundingProperty gp2 = new GroundingProperty();
				gp2
						.setConversationURI("http://www.dyonisos.org/MultimediaBrowser.owl#SetOnConversation");
				gp2
						.setOperationName("http://www.dyonisos.org/MultimediaBrowser.wsdl#setOn");
				gp2
						.setOperationPortType("http://www.dyonisos.org/MultimediaBrowser.wsdl#MultimediaBrowser");
				gp2
						.setWsdlFileName("http://www.dyonisos.org/MultimediaBrowser.wsdl");
				gp2
						.setWsdlInputMessage("http://www.dyonisos.org/MultimediaBrowser.wsdl#setOnRequest");
				gp2
						.setWsdlOutputMessage("http://www.dyonisos.org/MultimediaBrowser.wsdl#SetOnResponse");

				c3.setGrounding(gp);
				c2.setGrounding(gp2);

				// s2.print(System.out);
				// s3.print(System.out);

				/* test */

				assertEquals("2 test output BrowseContent",
						(new ArrayList<Output>(s3.getCapabilityFromTypeName("BrowseContent").get(0)
								.getOutputs().values()).get(0)), out);
				assertEquals("2 test output SetOn",
						(new ArrayList<Output>(s3.getCapabilityFromTypeName("SetOn").get(0)
								.getOutputs().values()).get(0)), out2);
				assertEquals("2 test capa 1", s3.getCapabilityFromTypeName("BrowseContent").get(0), c3);
				assertEquals("2 test capa 2", s3.getCapabilityFromTypeName("SetOn").get(0), c2);
				/*assertEquals("2 test output BrowseContent 2",
						(new ArrayList<Output>(s3.getProvidedCapabilities()
								.get(0).getOutputs().values()).get(0)), out2p);
				assertEquals("2 test output SetOn 2 ", (new ArrayList<Output>(
						s3.getProvidedCapabilities().get(0).getOutputs()
								.values()).get(1)), outp);
				assertEquals("2 test capa 3", s3.getProvidedCapabilities().get(
						0).getSubCapabilities().get(0), c2);
				assertEquals("2 test capa 4", s3.getProvidedCapabilities().get(
						0).getSubCapabilities().get(1), c3);*/

				provided = s3.getProvidedCapabilities().get(0);
				assertEquals("2 same type", provided.getType(), c.getType());
				assertEquals("2 same localName", provided.getLocalName(), c
						.getLocalName());
				assertEquals("2 same URI", provided.getCapabilityURI(), c
						.getCapabilityURI());
				assertEquals("2 same ConversationURI", provided
						.getConversationURI(), c.getConversationURI());
				assertEquals("2 same atomicity", provided.isAtomic(), c
						.isAtomic());
				assertEquals("2 same inputs", provided.getInputs(), c
						.getInputs());
				assertEquals("2 same outputs", provided.getOutputs(), c
						.getOutputs());
				assertEquals("2 same subcapa", provided.getSubCapabilities(), c
						.getSubCapabilities());
				assertTrue("2 test equals withoutSubcpabilities", provided
						.equalsWithoutSubCapabilities(c));

				assertEquals("test provided capa", provided, c);

				// test not null
				assertNotNull("test service not null (grounding)", s3);

				// test equals s2 s3
				assertEquals("test of equality of grounding property", gp, s3
						.getCapabilityFromTypeName("BrowseContent").get(0).getGrounding());
				assertEquals("test of equality of grounding property", gp2, s3
						.getCapabilityFromTypeName("SetOn").get(0).getGrounding());
				assertTrue("test of equality without Capability", s3
						.equalsWithoutCapabilities(s2));
				assertEquals("test of equality (grounding)", s3, s2);

			} else {
				fail("file " + SDLfilename
						+ " not found ! Test cannot be perform");
			}

		} catch (ParsingException e) {
			fail("Parsing Exception (" + e.getMessage() + ")");
		}

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.semanticcomposition.sdlparser.SDL2ServiceParser#parseProcess(java.lang.String, boolean)}.
	 */
	@Test
	public final void testParseProcessPiLowService() {
		String SDLfilename = System.getProperty("user.dir")
				+ "/amigo-s/PiLowService.owl";
		String SDLfilename2 = System.getProperty("user.dir")
				+ "/amigo-s/PiLowServiceWithoutQoS.owl";

		try {
			if ((new File(SDLfilename)).exists()
					&& (new File(SDLfilename2)).exists()) {

				// //////////////////////////
				// Test Without Grounding //
				// //////////////////////////

				/* get service from parsing */
				Long time = System.currentTimeMillis();
				Service s = ServiceParserFactory.getInstance().getParser().parseProcess(Tools
						.getURIStringFromFilePath(SDLfilename), false);
				time = System.currentTimeMillis() - time;
				DebugPrinter.print("\nparsing " + SDLfilename
						+ " time with QoS = " + time + " ms", 1);

				time = System.currentTimeMillis();
				Service s3 = ServiceParserFactory.getInstance().getParser().parseProcess(Tools
						.getURIStringFromFilePath(SDLfilename2), false);
				time = System.currentTimeMillis() - time;
				DebugPrinter.print("\nparsing " + SDLfilename2
						+ " time without QoS = " + time + " ms", 1);

				// s.print(System.out);

				/* Generate the equivalent service */

				Output out = new Output("PI",
						"http://www.thalesgroup.com/setha-dyonisos-demo.owl",
						"http://www.dyonisos.org/PiLowService.owl#EvaluatePiOutput");
				out.setProvided(true);

				Output out2 = new Output("PI",
						"http://www.thalesgroup.com/setha-dyonisos-demo.owl",
						"http://www.dyonisos.org/PiLowService.owl#GetPiValueAsDoubleOutput");
				out2.setProvided(true);

				Output out3 = new Output("PositiveRealNumber",
						"http://www.thalesgroup.com/setha-dyonisos-demo.owl",
						"http://www.dyonisos.org/PiLowService.owl#GetNumberOfDigitOutput");
				out3.setProvided(true);

				Capability c = new Capability();
				c
						.setConversationURI("http://www.dyonisos.org/PiLowService.owl#EvaluatePiConversation");
				c.setAtomic(true);
				c
						.setCapabilityURI("http://www.dyonisos.org/PiLowService.owl#EvaluatePiCapability");
				c
						.setType("http://www.thalesgroup.com/setha-dyonisos-demo.owl#PiCalculatingAsFloat");
				c.setLocalName("EvaluatePiCapability");
				c.addOutput(out);

				Capability c2 = new Capability();
				c2
						.setConversationURI("http://www.dyonisos.org/PiLowService.owl#GetPiValueAsDoubleConversation");
				c2.setAtomic(true);
				c2
						.setCapabilityURI("http://www.dyonisos.org/PiLowService.owl#GetPiValueAsDoubleCapability");
				c2
						.setType("http://www.thalesgroup.com/setha-dyonisos-demo.owl#PiCalculatingAsDouble");
				c2.setLocalName("GetPiValueAsDoubleCapability");
				c2.addOutput(out2);

				Capability c3 = new Capability();
				c3
						.setConversationURI("http://www.dyonisos.org/PiLowService.owl#GetNumberOfDigitConversation");
				c3.setAtomic(true);
				c3
						.setCapabilityURI("http://www.dyonisos.org/PiLowService.owl#GetNumberOfDigitCapability");
				c3
						.setType("http://www.thalesgroup.com/setha-dyonisos-demo.owl#PositiveRealNumber");
				c3.setLocalName("GetNumberOfDigitCapability");
				c3.addOutput(out3);

				Service s2 = new Service();
				s2.addProvidedCapability(c);
				s2.addProvidedCapability(c2);
				s2.addProvidedCapability(c3);
				s2.setBase("http://www.dyonisos.org/PiLowService.owl");
				s2.setLocalName("PiLowService");
				s2
						.setType("http://www.thalesgroup.com/setha-dyonisos-demo.owl#PiService");
				s2.setOwlDescriptionFile(Tools
						.getURIStringFromFilePath(SDLfilename2));
				c.setService(s2);
				c2.setService(s2);
				c3.setService(s2);

				/* test equal without Qos */
				assertEquals("same Type", s2.getType(), s3.getType());
				assertEquals("same owl file", s2.getOwlDescriptionFile(), s3
						.getOwlDescriptionFile());
				assertEquals("same LocalName", s2.getLocalName(), s3
						.getLocalName());
				assertEquals("same base", s2.getBase(), s3.getBase());
				assertTrue("same without Subcapabilities", s2
						.equalsWithoutCapabilities(s3));

				assertEquals(
						"same capa1",
						s3
								.getProvided_Capabilities()
								.get(
										"http://www.dyonisos.org/PiLowService.owl#GetNumberOfDigitCapability"),
						c3);
				assertEquals(
						"same capa2",
						s3
								.getProvided_Capabilities()
								.get(
										"http://www.dyonisos.org/PiLowService.owl#GetPiValueAsDoubleCapability"),
						c2);
				assertEquals(
						"same capa3",
						s3
								.getProvided_Capabilities()
								.get(
										"http://www.dyonisos.org/PiLowService.owl#EvaluatePiCapability"),
						c);

				assertTrue("same provided", Tools.compareMap(s2
						.getProvided_Capabilities(), s3
						.getProvided_Capabilities()));
				assertTrue("same required", Tools.compareMap(s2
						.getRequired_Capabilities(), s3
						.getRequired_Capabilities()));
				assertTrue("same other", Tools.compareMap(s2
						.getCapabilitiesMap(), s3.getCapabilitiesMap()));

				assertEquals("equals without QoS", s3, s2);

				/* test QosProperty not null */
				assertNotNull("qos not null", s.getQos());

			}
		} catch (ParsingException e) {
			fail("Parsing Exception (" + e.getMessage() + ")");
		}

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.semanticcomposition.sdlparser.SDL2ServiceParser#parseProcessFromString(java.lang.String, boolean)}.
	 */
	@Test
	public final void testParseProcessT_MultimediaPlayer_DB() {
		String SDLfilename = System.getProperty("user.dir")
				+ "/amigo-s/T_MultimediaPlayer_DB.owl";
		
		try {
			if ((new File(SDLfilename)).exists()
					&& (new File(SDLfilename)).exists()) {

				// //////////////////////////
				// Test Without Grounding //
				// //////////////////////////

				/* get service from parsing */
				Long time = System.currentTimeMillis();
				Service s = ServiceParserFactory.getInstance().getParser().parseProcess(Tools
						.getURIStringFromFilePath(SDLfilename), false);
				time = System.currentTimeMillis() - time;
				DebugPrinter.print("\nparsing " + SDLfilename
						+ " time without QoS = " + time + " ms", 1);
				
				//s.print(System.out);
				
			} else {
				fail("file " + SDLfilename
						+ " not found ! Test cannot be perform");
			}

		} catch (ParsingException e) {
			fail("Parsing Exception (" + e.getMessage() + ")");
		}

	}

}
