/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.junittest.servicemodel;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.HashMap;
import java.util.Map;

import org.junit.Test;

import fr.inria.icocoa.exception.CreationOfRecursionException;
import fr.inria.icocoa.exception.SemanticTypeNotKnowException;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.ControlConstruct;
import fr.inria.icocoa.servicemodel.GroundingProperty;
import fr.inria.icocoa.servicemodel.Input;
import fr.inria.icocoa.servicemodel.Output;
import fr.inria.icocoa.servicemodel.matching.CapabilityMatch;
import fr.inria.icocoa.servicemodel.matching.IOMatch;
import fr.inria.icocoa.servicemodel.matching.MatchingType;
import fr.inria.icocoa.servicemodel.matching.SyntacticMatcher;

/**
 * 
 * This class contains tests for the class
 * {@link fr.inria.icocoa.servicemodel.Capability}
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Apr 17, 2007 4:33:05 PM
 */
public class CapabilityTest {

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Capability#hashCode()}.
	 */
	@Test
	public final void testHashCode() {
		try {

			Input i1 = new Input("a", "b", "c");
			Input i2 = new Input("a", "b", "c");
			Input i3 = new Input("a2", "b", "c");
			Input i4 = new Input("e", "f", "g");

			Output o1 = new Output("a", "b", "c");
			Output o2 = new Output("a", "b", "c");
			Output o3 = new Output("a2", "b", "c");
			Output o4 = new Output("e", "f", "g");

			/*
			 * Test Capability without IO/SubCapability
			 */
			Capability c1 = new Capability("a#a", "b");
			Capability c2 = new Capability("a#a", "b");
			Capability c3 = new Capability("e#e", "b");

			// equals
			assertTrue("testEqualsObject : same object", c1.hashCode() == c1
					.hashCode());
			assertTrue("testEqualsObject : same semantic", c1.hashCode() == c2
					.hashCode());
			assertTrue("testEqualsObject : same semantic 2",
					c2.hashCode() == c1.hashCode());
			assertTrue("testEqualsObject : same reference", c1 == c1);

			// not equals
			assertTrue("testEqualsObject : not same reference", c1 != c2);
			assertFalse("testEqualsObject : not same semantic 1",
					c1.hashCode() == c3.hashCode());
			assertFalse("testEqualsObject : not same semantic 2",
					c2.hashCode() == c3.hashCode());
			assertFalse("testEqualsObject : not same semantic 3",
					c3.hashCode() == c1.hashCode());

			/*
			 * Test with Inputs
			 */
			Capability c11 = new Capability("a#a", "b");
			Capability c21 = new Capability("a#a", "b");
			Capability c31 = new Capability("a#a", "b");
			c11.addInput(i1);
			c11.addInput(i3);
			c21.addInput(i2);
			c21.addInput(i3);
			c31.addInput(i1);
			c31.addInput(i4);

			// equals
			assertTrue("testEqualsObject : same object (with I)", c11
					.hashCode() == c11.hashCode());
			assertTrue("testEqualsObject : same semantic (with I)", c11
					.hashCode() == c21.hashCode());
			assertTrue("testEqualsObject : same semantic (with I) 2", c21
					.hashCode() == c11.hashCode());

			// not equals
			assertFalse("testEqualsObject : not same semantic 1 (with I)", c11
					.hashCode() == c31.hashCode());
			assertFalse("testEqualsObject : not same semantic 2 (with I)", c21
					.hashCode() == c31.hashCode());
			assertFalse("testEqualsObject : not same semantic 3 (with I)", c31
					.hashCode() == c11.hashCode());

			/*
			 * Test with Outputs
			 */
			Capability c12 = new Capability("a#a", "b");
			Capability c22 = new Capability("a#a", "b");
			Capability c32 = new Capability("a#a", "b");
			c12.addOutput(o1);
			c12.addOutput(o3);
			c22.addOutput(o2);
			c22.addOutput(o3);
			c32.addOutput(o1);
			c32.addOutput(o4);

			// equals
			assertTrue("testEqualsObject : same object (with O)", c12
					.hashCode() == c12.hashCode());
			assertTrue("testEqualsObject : same semantic (with O)", c12
					.hashCode() == c22.hashCode());
			assertTrue("testEqualsObject : same semantic (with O) 2", c22
					.hashCode() == c12.hashCode());

			// not equals
			assertFalse("testEqualsObject : not same semantic 1 (with O)", c12
					.hashCode() == c32.hashCode());
			assertFalse("testEqualsObject : not same semantic 2 (with O)", c22
					.hashCode() == c32.hashCode());
			assertFalse("testEqualsObject : not same semantic 3 (with O)", c32
					.hashCode() == c12.hashCode());

			/*
			 * Test with GroundingProperty
			 */
			Capability c14 = new Capability("a#a", "b");
			Capability c24 = new Capability("a#a", "b");
			Capability c34 = new Capability("a#a", "b");

			GroundingProperty gp1 = new GroundingProperty();
			GroundingProperty gp2 = new GroundingProperty();
			gp1.setConversationURI("toto");
			gp2.setConversationURI("toto");
			GroundingProperty gp3 = new GroundingProperty();

			c14.setGrounding(gp1);
			c24.setGrounding(gp2);
			c34.setGrounding(gp3);

			// equals
			assertTrue("testEqualsObject : same object (with GP)", c14
					.hashCode() == c14.hashCode());
			assertTrue("testEqualsObject : same semantic (with GP)", c14
					.hashCode() == c24.hashCode());
			assertTrue("testEqualsObject : same semantic (with GP) 2", c24
					.hashCode() == c14.hashCode());

			// not equals
			assertFalse("testEqualsObject : not same semantic 1 (with GP)", c14
					.hashCode() == c34.hashCode());
			assertFalse("testEqualsObject : not same semantic 2 (with GP)", c24
					.hashCode() == c34.hashCode());
			assertFalse("testEqualsObject : not same semantic 3 (with GP)", c34
					.hashCode() == c14.hashCode());

			/*
			 * Test with SubCapabilities
			 */
			Capability c130 = new Capability("a#a", "b");
			Capability c230 = new Capability("a#a", "b");
			Capability c330 = new Capability("a#a", "b");
			Capability c430 = new Capability("a#a", "b");

			Capability sc10 = new Capability("x#x", "y");
			Capability sc20 = new Capability("x#x", "y");
			Capability sc30 = new Capability("z#x", "w");
			Capability sc40 = new Capability("a#a", "y");

			c130.addSubCapability_E(sc10);
			c130.addSubCapability_E(sc30);
			c230.addSubCapability_E(sc20);
			c230.addSubCapability_E(sc30);
			c330.addSubCapability_E(sc10);
			c330.addSubCapability_E(sc40);
			c430.addSubCapability_E(sc10);
			c430.addSubCapability_E(sc30);
			c430.addSubCapability_E(sc40);

			// equals
			assertTrue("testEqualsObject : same object (with SC)", c130
					.hashCode() == c130.hashCode());
			assertTrue("testEqualsObject : same semantic (with SC)", c130
					.hashCode() == c230.hashCode());
			assertTrue("testEqualsObject : same semantic (with SC) 2", c230
					.hashCode() == c130.hashCode());

			// not equals
			assertFalse("testEqualsObject : not same semantic 1 (with SC)",
					c130.hashCode() == c330.hashCode());
			assertFalse("testEqualsObject : not same semantic 2 (with SC)",
					c230.hashCode() == c330.hashCode());
			assertFalse("testEqualsObject : not same semantic 3 (with SC)",
					c330.hashCode() == c130.hashCode());
			assertFalse("testEqualsObject : not same semantic 3 (with + SC)",
					c130.hashCode() == c430.hashCode());

		} catch (CreationOfRecursionException e) {
			fail("An unexpeted exception as been raised !");
		}
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Capability#equals(java.lang.Object)}.
	 */
	@Test
	public final void testEqualsObject() {

		try {
			/*
			 * Test with SubCapabilities
			 */
			Capability c13 = new Capability("a#a", "b");
			Capability c23 = new Capability("a#a", "b");
			Capability c33 = new Capability("a#a", "b");
			Capability c43 = new Capability("a#a", "b");

			Capability sc1 = new Capability("x#x", "y");
			Capability sc2 = new Capability("x#x", "y");
			Capability sc3 = new Capability("z#z", "w");
			Capability sc4 = new Capability("a#a", "y");

			c13.addSubCapability_E(sc1);
			c13.addSubCapability_E(sc3);
			c23.addSubCapability_E(sc2);
			c23.addSubCapability_E(sc3);
			c33.addSubCapability_E(sc1);
			c33.addSubCapability_E(sc4);
			c43.addSubCapability_E(sc1);
			c43.addSubCapability_E(sc3);
			c43.addSubCapability_E(sc4);

			// equals
			assertTrue("testEqualsObject : same object (with SC)", c13
					.equals(c13));
			assertTrue("testEqualsObject : same semantic (with SC)", c13
					.equals(c23));
			assertTrue("testEqualsObject : same semantic (with SC) 2", c23
					.equals(c13));

			// not equals
			assertFalse("testEqualsObject : not same semantic 1 (with SC)", c13
					.equals(c33));
			assertFalse("testEqualsObject : not same semantic 2 (with SC)", c23
					.equals(c33));
			assertFalse("testEqualsObject : not same semantic 3 (with SC)", c33
					.equals(c13));
			assertFalse("testEqualsObject : not same semantic 3 (with + SC)",
					c13.equals(c43));
		} catch (CreationOfRecursionException e) {
			fail("An unexpeted exception as been raised !");
		}

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Capability#equals(java.lang.Object)}.
	 */
	@Test
	public final void testEqualsObjectWithoutSubCapabilities() {
		try {

			Input i1 = new Input("a", "b", "c");
			Input i2 = new Input("a", "b", "c");
			Input i3 = new Input("a2", "b", "c");
			Input i4 = new Input("e", "f", "g");

			Output o1 = new Output("a", "b", "c");
			Output o2 = new Output("a", "b", "c");
			Output o3 = new Output("a2", "b", "c");
			Output o4 = new Output("e", "f", "g");

			/*
			 * Test Capability without IO/SubCapability
			 */
			Capability c1 = new Capability("a#a", "b");
			Capability c2 = new Capability("a#a", "b");
			Capability c3 = new Capability("e#e", "b");

			Capability c100 = new Capability("toto#toto");

			// test equivalence c1 and new Capa with same name
			assertTrue("test equivalence", c100
					.equalsWithoutSubCapabilities(new Capability("toto#toto")));

			// equals
			assertTrue("testEqualsObject : same object", c1
					.equalsWithoutSubCapabilities(c1));
			assertTrue("testEqualsObject : same semantic", c1
					.equalsWithoutSubCapabilities(c2));
			assertTrue("testEqualsObject : same semantic 2", c2
					.equalsWithoutSubCapabilities(c1));
			assertTrue("testEqualsObject : same reference", c1 == c1);

			// not equals
			assertTrue("testEqualsObject : not same reference", c1 != c2);
			assertFalse("testEqualsObject : not same semantic 1", c1
					.equalsWithoutSubCapabilities(c3));
			assertFalse("testEqualsObject : not same semantic 2", c2
					.equalsWithoutSubCapabilities(c3));
			assertFalse("testEqualsObject : not same semantic 3", c3
					.equalsWithoutSubCapabilities(c1));

			/*
			 * Test with Inputs
			 */
			Capability c11 = new Capability("a#a", "b");
			Capability c21 = new Capability("a#a", "b");
			Capability c31 = new Capability("a#a", "b");
			c11.addInput(i1);
			c11.addInput(i3);
			c21.addInput(i2);
			c21.addInput(i3);
			c31.addInput(i1);
			c31.addInput(i4);

			// equals
			assertTrue("testEqualsObject : same object (with I)", c11
					.equalsWithoutSubCapabilities(c11));
			assertTrue("testEqualsObject : same semantic (with I)", c11
					.equalsWithoutSubCapabilities(c21));
			assertTrue("testEqualsObject : same semantic (with I) 2", c21
					.equalsWithoutSubCapabilities(c11));

			// not equals
			assertFalse("testEqualsObject : not same semantic 1 (with I)", c11
					.equalsWithoutSubCapabilities(c31));
			assertFalse("testEqualsObject : not same semantic 2 (with I)", c21
					.equalsWithoutSubCapabilities(c31));
			assertFalse("testEqualsObject : not same semantic 3 (with I)", c31
					.equalsWithoutSubCapabilities(c11));

			/*
			 * Test with Outputs
			 */
			Capability c12 = new Capability("a#a", "b");
			Capability c22 = new Capability("a#a", "b");
			Capability c32 = new Capability("a#a", "b");
			c12.addOutput(o1);
			c12.addOutput(o3);
			c22.addOutput(o2);
			c22.addOutput(o3);
			c32.addOutput(o1);
			c32.addOutput(o4);

			// equals
			assertTrue("testEqualsObject : same object (with O)", c12
					.equalsWithoutSubCapabilities(c12));
			assertTrue("testEqualsObject : same semantic (with O)", c12
					.equalsWithoutSubCapabilities(c22));
			assertTrue("testEqualsObject : same semantic (with O) 2", c22
					.equalsWithoutSubCapabilities(c12));

			// not equals
			assertFalse("testEqualsObject : not same semantic 1 (with O)", c12
					.equalsWithoutSubCapabilities(c32));
			assertFalse("testEqualsObject : not same semantic 2 (with O)", c22
					.equalsWithoutSubCapabilities(c32));
			assertFalse("testEqualsObject : not same semantic 3 (with O)", c32
					.equalsWithoutSubCapabilities(c12));

			/*
			 * Test with SubCapabilities
			 */
			Capability c13 = new Capability("a#a", "b");
			Capability c23 = new Capability("a#a", "b");
			Capability c33 = new Capability("a#a", "b");

			Capability sc1 = new Capability("x#x", "y");
			Capability sc2 = new Capability("x#x", "y");
			Capability sc3 = new Capability("z#z", "w");
			Capability sc4 = new Capability("a#a", "y");

			c13.addSubCapability_E(sc1);
			c13.addSubCapability_E(sc3);
			c23.addSubCapability_E(sc2);
			c23.addSubCapability_E(sc3);
			c33.addSubCapability_E(sc1);
			c33.addSubCapability_E(sc4);

			// equals
			assertTrue("testEqualsObject : same object (with SC)", c13
					.equalsWithoutSubCapabilities(c13));
			assertTrue("testEqualsObject : same semantic (with SC)", c13
					.equalsWithoutSubCapabilities(c23));
			assertTrue("testEqualsObject : same semantic (with SC) 2", c23
					.equalsWithoutSubCapabilities(c13));

			// not equals but return equals --> Ok because don't condider the
			// Subcapabilities
			assertTrue("testEqualsObject : not same semantic 1 (with SC)", c13
					.equalsWithoutSubCapabilities(c33));
			assertTrue("testEqualsObject : not same semantic 2 (with SC)", c23
					.equalsWithoutSubCapabilities(c33));
			assertTrue("testEqualsObject : not same semantic 3 (with SC)", c33
					.equalsWithoutSubCapabilities(c13));

			/*
			 * Test with GroundingProperty
			 */
			Capability c14 = new Capability("a#a", "b");
			Capability c24 = new Capability("a#a", "b");
			Capability c34 = new Capability("a#a", "b");

			GroundingProperty gp1 = new GroundingProperty();
			GroundingProperty gp2 = new GroundingProperty();
			gp1.setConversationURI("toto");
			gp2.setConversationURI("toto");
			GroundingProperty gp3 = new GroundingProperty();

			c14.setGrounding(gp1);
			c24.setGrounding(gp2);
			c34.setGrounding(gp3);

			// equals
			assertTrue("testEqualsObject : same object (with GP)", c14
					.equalsWithoutSubCapabilities(c14));
			assertTrue("testEqualsObject : same semantic (with GP)", c14
					.equalsWithoutSubCapabilities(c24));
			assertTrue("testEqualsObject : same semantic (with GP) 2", c24
					.equalsWithoutSubCapabilities(c14));

			// not equals
			assertFalse("testEqualsObject : not same semantic 1 (with GP)", c14
					.equalsWithoutSubCapabilities(c34));
			assertFalse("testEqualsObject : not same semantic 2 (with GP)", c24
					.equalsWithoutSubCapabilities(c34));
			assertFalse("testEqualsObject : not same semantic 3 (with GP)", c34
					.equalsWithoutSubCapabilities(c14));

		} catch (CreationOfRecursionException e) {
			fail("An unexpeted exception as been raised !");
		}

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Capability#getSubCapabilityFromTypeName(java.lang.String)}.
	 */
	@Test
	public final void testGetSubCapabilityNamed() {
		try {
			Capability c = new Capability("a#a", "b");
			Capability c2 = new Capability("a#a", "b");
			Capability c3 = new Capability("a#a", "b");
			Capability c4 = new Capability("a#a", "b");

			Capability sc1 = new Capability("x#x", "y");
			Capability sc2 = new Capability("x#x", "y");
			Capability sc3 = new Capability("z#z", "w");
			Capability sc4 = new Capability("a#a", "y");

			c.addSubCapability_E(sc1);
			c.addSubCapability_E(sc2);
			c.addSubCapability_E(sc3);
			c.addSubCapability_E(sc4);

			c2.addSubCapability_E(sc2);
			c2.addSubCapability_E(sc1);

			c3.addSubCapability_E(sc1);
			c3.addSubCapability_E(sc3);
			c3.addSubCapability_E(sc4);

			/* Test access */
			assertEquals("get sc1", sc1, c.getSubCapabilityFromTypeName("x"));
			assertEquals("get sc2", sc2, c.getSubCapabilityFromTypeName("x"));
			assertEquals("get sc3", sc3, c.getSubCapabilityFromTypeName("z"));
			assertEquals("get sc4", sc4, c.getSubCapabilityFromTypeName("a"));

			/* Test order */
			assertTrue("get sc1 object",
					c.getSubCapabilityFromTypeName("x") == sc1);
			assertFalse("get sc1 object",
					c.getSubCapabilityFromTypeName("x") == sc2);

			assertFalse("get sc1 object",
					c2.getSubCapabilityFromTypeName("x") == sc1);
			assertTrue("get sc1 object",
					c2.getSubCapabilityFromTypeName("x") == sc2);

			/* Test return null */
			assertNull("get a non existing subCapability", c3
					.getSubCapabilityFromTypeName("toto"));
			assertNull("get a object in a empty list of SubCapability", c4
					.getSubCapabilityFromTypeName("toto"));
		} catch (CreationOfRecursionException e) {
			fail("An unexpeted exception as been raised !");
		}

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Capability#addSubCapability_E(fr.inria.icocoa.servicemodel.Capability)}.
	 * 
	 * @throws CreationOfRecursionException
	 */
	@Test(expected = fr.inria.icocoa.exception.CreationOfRecursionException.class)
	public final void testAddSubCapability_E1()
			throws CreationOfRecursionException {
		Capability c = new Capability("a#a", "b");
		Capability sc1 = new Capability("x#x", "y");
		Capability sc2 = new Capability("x#x", "y");
		try {
			c.addSubCapability_E(sc1);
		} catch (CreationOfRecursionException e) {
			fail("An unexpeted exception as been raised ! (sc1)");
		}
		try {
			c.addSubCapability_E(sc1);
		} catch (CreationOfRecursionException e) {
			fail("An unexpeted exception as been raised ! (+sc1)");
		}
		try {
			c.addSubCapability_E(sc2);
		} catch (CreationOfRecursionException e) {
			fail("An unexpeted exception as been raised ! (+sc2)");
		}
		try {
			c.addSubCapability_E(c);
		} catch (CreationOfRecursionException e) {
			// this time it's ok ;)
			throw e;
		}

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Capability#addSubCapability_E(fr.inria.icocoa.servicemodel.Capability)}.
	 * 
	 * @throws CreationOfRecursionException
	 */
	@Test(expected = fr.inria.icocoa.exception.CreationOfRecursionException.class)
	public final void testAddSubCapability_E2()
			throws CreationOfRecursionException {
		Capability c = new Capability("a#a", "b");
		try {
			c.addSubCapability_E(new Capability("a#a", "b"));
		} catch (CreationOfRecursionException e) {
			// this time it's ok ;)
			throw e;
		}

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Capability#addSubCapability(fr.inria.icocoa.servicemodel.Capability)}.
	 * 
	 */
	@Test
	public final void testAddSubCapability() {
		Capability c = new Capability("a#a", "b");
		Capability sc1 = new Capability("x#x", "y");
		Capability sc2 = new Capability("x#x", "y");

		assertTrue("first addition", c.addSubCapability(sc1));
		assertTrue("verification 1 of nb of subcapabiliy (add ok)", c
				.getSubCapabilities().size() == 1);
		assertTrue("same addition", c.addSubCapability(sc1));
		assertTrue("verification 2 of nb of subcapabiliy (add ok)", c
				.getSubCapabilities().size() == 2);
		assertTrue("addition an other object", c.addSubCapability(sc2));
		assertTrue("verification 3 of nb of subcapabiliy (add ok)", c
				.getSubCapabilities().size() == 3);

		assertFalse("add current object => Creation of recursion", c
				.addSubCapability(c));
		assertFalse("verification 4 of nb of subcapabiliy (add ok)", c
				.getSubCapabilities().size() == 4);
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Capability#setType(java.lang.String)}.
	 */
	@Test
	public final void testSetType() {
		Capability in = new Capability("toto#toto", "tata");
		in.setType("blabla#bloblo");
		assertEquals("test localName", "tata", in.getLocalName());
		assertEquals("test Name", "bloblo", in.getTypeName());
		assertEquals("test Ontology", "blabla", in.getOntology());
		assertEquals("test getType", "blabla#bloblo", in.getType());
	}

	/**
	 * Test method for {@link fr.inria.icocoa.servicemodel.Capability#getType()}.
	 */
	@Test
	public final void testGetType() {
		Capability in = new Capability("titi#toto", null, "tata");
		assertEquals("test getType", "titi#toto", in.getType());
		assertEquals("test localName", "tata", in.getLocalName());
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Capability#ioExactMatch(fr.inria.icocoa.servicemodel.Capability)}.
	 * 
	 * @throws SemanticTypeNotKnowException
	 */
	/*
	 * @Test public final void testIoExactMatch() throws
	 * SemanticTypeNotKnowException { // Reference Capa Input in1 = new
	 * Input("in1", "http://www.inria.fr/myPiOntology.owl#SemIn1"); Input in2 =
	 * new Input("in2", "http://www.inria.fr/myPiOntology.owl#SemIn2"); Input
	 * in3 = new Input("in3", "http://www.inria.fr/myPiOntology.owl#SemIn3");
	 * 
	 * Output out1 = new Output("out1",
	 * "http://www.inria.fr/myPiOntology.owl#SemOut1"); Output out2 = new
	 * Output("out2", "http://www.inria.fr/myPiOntology.owl#SemOut2"); Output
	 * out3 = new Output("out3",
	 * "http://www.inria.fr/myPiOntology.owl#SemOut3");
	 * 
	 * Capability capa1 = new Capability(
	 * "http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa1");
	 * 
	 * capa1.addInput(in1); capa1.addInput(in2); capa1.addInput(in3);
	 * 
	 * capa1.addOutput(out1); capa1.addOutput(out2); capa1.addOutput(out3); //
	 * Other Capa Input in11 = new Input("in11",
	 * "http://www.inria.fr/myPiOntology.owl#SemIn1"); Input in21 = new
	 * Input("in21", "http://www.inria.fr/myPiOntology.owl#SemIn2"); Input in31 =
	 * new Input("in31", "http://www.inria.fr/myPiOntology.owl#SemIn3");
	 * 
	 * Output out11 = new Output("out11",
	 * "http://www.inria.fr/myPiOntology.owl#SemOut1"); Output out21 = new
	 * Output("out21", "http://www.inria.fr/myPiOntology.owl#SemOut2"); Output
	 * out31 = new Output("out31",
	 * "http://www.inria.fr/myPiOntology.owl#SemOut3");
	 * 
	 * Capability capa11 = new Capability(
	 * "http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa11");
	 * 
	 * capa11.addInput(in11); capa11.addInput(in21); capa11.addInput(in31);
	 * 
	 * capa11.addOutput(out11); capa11.addOutput(out21);
	 * capa11.addOutput(out31);
	 * 
	 * Matcher matcher = new SyntaxicMatcher(); // test notNull
	 * assertNotNull("test not null OK", capa11.match(capa1, matcher)); // Other
	 * Capa (less input) Input in111 = new Input("in111",
	 * "http://www.inria.fr/myPiOntology.owl#SemIn1"); Input in211 = new
	 * Input("in211", "http://www.inria.fr/myPiOntology.owl#SemIn2");
	 * 
	 * Output out111 = new Output("out111",
	 * "http://www.inria.fr/myPiOntology.owl#SemOut1"); Output out211 = new
	 * Output("out211", "http://www.inria.fr/myPiOntology.owl#SemOut2"); Output
	 * out311 = new Output("out311",
	 * "http://www.inria.fr/myPiOntology.owl#SemOut3");
	 * 
	 * Capability capa111 = new Capability(
	 * "http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa111");
	 * 
	 * capa111.addInput(in111); capa111.addInput(in211);
	 * 
	 * capa111.addOutput(out111); capa111.addOutput(out211);
	 * capa111.addOutput(out311); // tests assertNull("test null (less input)",
	 * capa111.match(capa1, matcher)); assertNotNull("test not null (more
	 * input)", capa1.match(capa111, matcher)); // Other Capa (less output)
	 * Input in1111 = new Input("in1111",
	 * "http://www.inria.fr/myPiOntology.owl#SemIn1"); Input in2111 = new
	 * Input("in2111", "http://www.inria.fr/myPiOntology.owl#SemIn2"); Input
	 * in3111 = new Input("in3111",
	 * "http://www.inria.fr/myPiOntology.owl#SemIn3");
	 * 
	 * Output out1111 = new Output("out1111",
	 * "http://www.inria.fr/myPiOntology.owl#SemOut1"); Output out2111 = new
	 * Output("out2111", "http://www.inria.fr/myPiOntology.owl#SemOut2");
	 * 
	 * Capability capa1111 = new Capability(
	 * "http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa1111");
	 * 
	 * capa1111.addInput(in1111); capa1111.addInput(in2111);
	 * capa1111.addInput(in3111);
	 * 
	 * capa1111.addOutput(out1111); capa1111.addOutput(out2111); // test null
	 * assertNotNull("test not null (less output)", capa1111.match(capa1,
	 * matcher)); assertNull("test null (more input)", capa1.match(capa1111,
	 * matcher)); // Other Capa (different Sem input) Input in12 = new
	 * Input("in12", "http://www.inria.fr/myPiOntology.owl#SemIn1"); Input in22 =
	 * new Input("in22", "http://www.inria.fr/myPiOntology.owl#SemIn2"); Input
	 * in32 = new Input("in32", "http://www.inria.fr/myPiOntology.owl#SemIn4");
	 * 
	 * Output out12 = new Output("out12",
	 * "http://www.inria.fr/myPiOntology.owl#SemOut1"); Output out22 = new
	 * Output("out22", "http://www.inria.fr/myPiOntology.owl#SemOut2"); Output
	 * out32 = new Output("out32",
	 * "http://www.inria.fr/myPiOntology.owl#SemOut3");
	 * 
	 * Capability capa12 = new Capability(
	 * "http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa12");
	 * 
	 * capa12.addInput(in12); capa12.addInput(in22); capa12.addInput(in32);
	 * 
	 * capa12.addOutput(out12); capa12.addOutput(out22);
	 * capa12.addOutput(out32); // test assertNull("different semantic input",
	 * capa1.match(capa12, matcher)); // Other Capa (different Sem output) Input
	 * in121 = new Input("in121",
	 * "http://www.inria.fr/myPiOntology.owl#SemIn1"); Input in221 = new
	 * Input("in221", "http://www.inria.fr/myPiOntology.owl#SemIn2"); Input
	 * in321 = new Input("in321",
	 * "http://www.inria.fr/myPiOntology.owl#SemIn3");
	 * 
	 * Output out121 = new Output("out121",
	 * "http://www.inria.fr/myPiOntology.owl#SemOut1"); Output out221 = new
	 * Output("out221", "http://www.inria.fr/myPiOntology.owl#SemOut2"); Output
	 * out321 = new Output("out321",
	 * "http://www.inria.fr/myPiOntology.owl#SemOut4");
	 * 
	 * Capability capa121 = new Capability(
	 * "http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa121");
	 * 
	 * capa121.addInput(in121); capa121.addInput(in221);
	 * capa121.addInput(in321);
	 * 
	 * capa121.addOutput(out121); capa121.addOutput(out221);
	 * capa121.addOutput(out321); // test assertNull("different semantic
	 * output", capa1.match(capa121, matcher)); // create equivalent table Map<IO,
	 * IO> matching = new HashMap<IO, IO>(); matching.put(in11, in1);
	 * matching.put(in21, in2); matching.put(in31, in3); matching.put(out11,
	 * out1); matching.put(out21, out2); matching.put(out31, out3);
	 * 
	 * DebugPrinter.print(matching, "matching = ", 1);
	 * DebugPrinter.print("capa11.match(capa1,matcher) = " + capa11.match(capa1,
	 * matcher)., 1); // test equivalent assertEquals("test same matching
	 * table", matching, capa11.match(capa1, matcher)); }
	 */

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Capability#ioExactMatch(fr.inria.icocoa.servicemodel.Capability)}.
	 * 
	 * @throws SemanticTypeNotKnowException
	 */
	@Test
	public final void testExactMatch() throws SemanticTypeNotKnowException {

		// Reference Capa
		Input in1 = new Input("in1",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in2 = new Input("in2",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in3 = new Input("in3",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out1 = new Output("out1",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out2 = new Output("out2",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out3 = new Output("out3",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa1 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa1");

		capa1.addInput(in1);
		capa1.addInput(in2);
		capa1.addInput(in3);

		capa1.addOutput(out1);
		capa1.addOutput(out2);
		capa1.addOutput(out3);

		// Other Capa
		Input in11 = new Input("in11",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in21 = new Input("in21",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in31 = new Input("in31",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out11 = new Output("out11",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out21 = new Output("out21",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out31 = new Output("out31",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa11 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa11");

		capa11.addInput(in11);
		capa11.addInput(in21);
		capa11.addInput(in31);

		capa11.addOutput(out11);
		capa11.addOutput(out21);
		capa11.addOutput(out31);

		CapabilityMatch matching = new SyntacticMatcher().capabilityMatch(capa1, capa11);
		// DebugPrinter.print("matching\n" + matching.toString(), 1);

		// expected result
		CapabilityMatch expected = new CapabilityMatch(capa11,capa1, MatchingType.EXACT);
		expected.addIOMatching(new IOMatch(in11,in1,MatchingType.EXACT));
		expected.addIOMatching(new IOMatch(in21,in2,MatchingType.EXACT));
		expected.addIOMatching(new IOMatch(in31,in3,MatchingType.EXACT));
		
		expected.addIOMatching(new IOMatch(out11,out1,MatchingType.EXACT));
		expected.addIOMatching(new IOMatch(out21,out2,MatchingType.EXACT));
		expected.addIOMatching(new IOMatch(out31,out3,MatchingType.EXACT));
		// DebugPrinter.print("expected\n" + expected.toString(), 1);

		// test notNull
		assertNotNull("test not null OK", matching);

		// testEquals
		assertEquals("test equals", matching, expected);
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Capability#isUsing(fr.inria.icocoa.servicemodel.IO)}.
	 * 
	 * @todo test to implements
	 */
	//@Test
	public final void testIsUsing() {
		// TODO test to implements
		fail("Not yet implemented"); // TODO
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Capability#getFirstCapaUsingIO(fr.inria.icocoa.servicemodel.IO)}
	 * 
	 * @todo test to implements
	 */
	//@Test
	public final void testGetFirstCapaUsingIO() {
		// TODO test to implements
		fail("Not yet implemented"); // TODO
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Capability#formatInvokation(java.util.Map)}
	 * 
	 */
	@Test
	public final void testFormatInvokation() {

		String s1 = "?method=operation1&gv2=value2&gv1=value1";
		String errorNull = "ERROR - null";
		String errorParam = "ERROR - uncomplete list of param";

		// Capability ok
		Input i1 = new Input("a", "b", "in1");
		i1.setGroundingValue("gv1");

		Input i2 = new Input("e", "f", "in2");
		i2.setGroundingValue("gv2");

		Capability c11 = new Capability("h#i", "j");
		c11.addInput(i1);
		c11.addInput(i2);

		GroundingProperty gp1 = new GroundingProperty();
		gp1.setConversationURI("toto");
		gp1.setWsdlFileName("blabla");
		gp1.setOperationName("operation1");

		c11.setGrounding(gp1);

		// Capability without grounding 1
		Input i11 = new Input("a", "b", "in1");
		i11.setGroundingValue("gv1");

		Input i21 = new Input("e", "f", "in2");
		i21.setGroundingValue("gv2");

		Capability c12 = new Capability("h#i", "j");
		c12.addInput(i11);
		c12.addInput(i21);

		// Capability without grounding 2
		Input i12 = new Input("a", "b", "in1");

		Input i22 = new Input("e", "f", "in2");
		i22.setGroundingValue("gv2");

		Capability c13 = new Capability("h#i", "j");
		c13.addInput(i12);
		c13.addInput(i22);

		GroundingProperty gp12 = new GroundingProperty();
		gp1.setConversationURI("toto");
		gp1.setWsdlFileName("blabla");
		gp1.setOperationName("operation1");

		c13.setGrounding(gp12);

		// list of params ok
		Map<String, String> params = new HashMap<String, String>();
		params.put("in1", "value1");
		params.put("in2", "value2");

		// list of params !ok
		Map<String, String> params2 = new HashMap<String, String>();
		params.put("in1", "value1");

		// System.out.println(c11.formatInvokation(params));

		// tests
		assertEquals("test format ok", s1, c11.formatInvokation(params));
		assertTrue("test miss a param", c11.formatInvokation(params2)
				.startsWith(errorParam));
		assertTrue("test no grounding", c12.formatInvokation(params)
				.startsWith(errorNull));
		assertTrue("test no grounding 2", c13.formatInvokation(params)
				.startsWith(errorNull));
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Capability#transfertInformation(fr.inria.icocoa.servicemodel.Capability)}.
	 */
	@Test
	public final void testTransfertInformation() {

		Capability a = new Capability("a#a", "a");
		Capability b = new Capability("myOntoilogy#myConceptName",
				"mylocalName");

		b.setCapabilityURI("capabilityURI");
		b.setCC(ControlConstruct.SEQUENCE);
		b.setConversationURI("conversationURI");
		b.setGrounding(new GroundingProperty());
		b.setTypeOfCapability(Capability.PROVIDED);

		assertFalse("Capability not equals", a.equals(b));

		a.transfertInformation(b);
		assertTrue("Capability equals", a.equals(b));
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Capability#countNullOrUnknowField()}.
	 */
	@Test
	public final void testCountNullOrUnknowField() {

		assertEquals("test Capability 1", 5, new Capability("a#a", "a")
				.countNullOrUnknowField());

		Capability a = new Capability("a#a", "a");
		Capability b = new Capability("myOntoilogy#myConceptName",
				"mylocalName");

		b.setCapabilityURI("capabilityURI");
		b.setCC(ControlConstruct.SEQUENCE);
		b.setConversationURI("conversationURI");
		b.setGrounding(new GroundingProperty());
		b.setTypeOfCapability(Capability.PROVIDED);

		assertEquals("test Capability 2", 1, b.countNullOrUnknowField());

		a.setLocalName(Capability.UNKNOWN);
		a.setType(Capability.UNKNOWN + "#" + Capability.UNKNOWN);

		assertEquals("test Capability 3", 8, a.countNullOrUnknowField());

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Capability#containsMoreInformation(fr.inria.icocoa.servicemodel.Capability)}.
	 */
	@Test
	public final void testContainsMoreInformation() {

		Capability a = new Capability("a#a", "a");
		Capability b = new Capability("myOntoilogy#myConceptName",
				"mylocalName");

		b.setCapabilityURI("capabilityURI");
		b.setCC(ControlConstruct.SEQUENCE);
		b.setConversationURI("conversationURI");
		b.setGrounding(new GroundingProperty());
		b.setTypeOfCapability(Capability.PROVIDED);

		assertEquals("test Capability equals", 0, new Capability("a#a", "a")
				.containsMoreInformation(new Capability("a#a", "a")));

		assertTrue("test Capability <0", a.containsMoreInformation(b) < 0);
		assertTrue("test Capability >0", b.containsMoreInformation(a) > 0);

	}
}
