/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.junittest.servicemodel.matching;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

import fr.inria.icocoa.exception.SemanticTypeNotKnowException;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.Input;
import fr.inria.icocoa.servicemodel.Output;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.UserTask;
import fr.inria.icocoa.servicemodel.matching.CapabilityMatch;
import fr.inria.icocoa.servicemodel.matching.IOMatch;
import fr.inria.icocoa.servicemodel.matching.Matcher;
import fr.inria.icocoa.servicemodel.matching.MatchingType;
import fr.inria.icocoa.servicemodel.matching.ServiceMatch;
import fr.inria.icocoa.servicemodel.matching.SyntacticMatcher;

/**
 * SyntaxicMatcherTest.java
 *
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Sep 12, 2007 11:32:09 AM
 */
public class SyntaxicMatcherTest {



	private Matcher matcher = new SyntacticMatcher();
	
	

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.IO#exactMatch(java.lang.String)}.
	 * 
	 * @throws SemanticTypeNotKnowException
	 */
	@Test
	public final void testExactMatch() throws SemanticTypeNotKnowException {
		Input in1 = new Input("b1", "http://www.inria.fr/myPiOntology.owl#PI");
		assertTrue("test exact match true", in1.match(
				"http://www.inria.fr/myPiOntology.owl#PI", matcher) == MatchingType.EXACT);

	}
	

	/**
	 * Test method for {@link fr.inria.icocoa.servicemodel.matching.SyntaxicMatcher#ioMatch(fr.inria.icocoa.servicemodel.IO, fr.inria.icocoa.servicemodel.IO)}.
	 * @throws SemanticTypeNotKnowException 
	 */
	@Test
	public final void testIoMatchIOIO() throws SemanticTypeNotKnowException {
		/*
		 * Test Input Match
		 */
		
		Input in1 = new Input("b1", "http://www.inria.fr/myPiOntology.owl#PI");
		Input in2 = new Input("b11", "http://www.inria.fr/myPiOntology.owl#PI");
		
				
		IOMatch inputMatch = matcher.ioMatch(in1, in2);
		
		IOMatch expectedInputMatch = new IOMatch(in2,in1,MatchingType.EXACT);
		
		assertNotNull("inputMatch not null",inputMatch);
		
		assertEquals("test expected inputMatch",inputMatch,expectedInputMatch);
		
		/*
		 * Test Output Match
		 */
		
		Output out1 = new Output("b1", "http://www.inria.fr/myPiOntology.owl#PI");
		Output out2 = new Output("b11", "http://www.inria.fr/myPiOntology.owl#PI");
		
		
		IOMatch outputMatch = matcher.ioMatch(out1, out2);
			
		
		IOMatch expectedOutputMatch = new IOMatch(out2,out1,MatchingType.EXACT);
		
		assertNotNull("outputMatch not null",outputMatch);
		
		assertEquals("test expected outputMatch",outputMatch,expectedOutputMatch);
		
		/*
		 * Test heterogenous matching 
		 */
		
		IOMatch inOutMatch = matcher.ioMatch(in1, out1);
		
		IOMatch outInMatch = matcher.ioMatch(out1, in1);
		
		//DebugPrinter.print("inOutMatch = "+ inOutMatch,DebugPrinter.INFO);
		//DebugPrinter.print("outInMatch = "+ outInMatch,DebugPrinter.INFO);
		
		assertNull("test null InOutMatch",inOutMatch);
		assertNull("test null OutInMatch",outInMatch);
	}
	

	/**
	 * Test method for matching with ExactSemanticMatcher (Capability Level)
	 * 
	 * @throws SemanticTypeNotKnowException
	 */
	@Test
	public final void testExactMatchAtCapabilityLevel()
			throws SemanticTypeNotKnowException {

		/* Capability Level */

		// Reference Capa
		Input in1 = new Input("in1",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in2 = new Input("in2",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in3 = new Input("in3",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out1 = new Output("out1",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out2 = new Output("out2",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out3 = new Output("out3",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa1 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa1");

		capa1.addInput(in1);
		capa1.addInput(in2);
		capa1.addInput(in3);

		capa1.addOutput(out1);
		capa1.addOutput(out2);
		capa1.addOutput(out3);

		// Other Capa
		Input in11 = new Input("in11",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in21 = new Input("in21",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in31 = new Input("in31",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out11 = new Output("out11",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out21 = new Output("out21",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out31 = new Output("out31",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa11 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa11");

		capa11.addInput(in11);
		capa11.addInput(in21);
		capa11.addInput(in31);

		capa11.addOutput(out11);
		capa11.addOutput(out21);
		capa11.addOutput(out31);

		CapabilityMatch matching = matcher.capabilityMatch(capa1,capa11); 
		// DebugPrinter.print("matching\n" + matching.toString(), 1);

		// expected result
		CapabilityMatch expected = new CapabilityMatch(capa11,capa1, MatchingType.EXACT);
		
		expected.addIOMatching(new IOMatch(in11,in1,MatchingType.EXACT));
		expected.addIOMatching(new IOMatch(in21,in2,MatchingType.EXACT));
		expected.addIOMatching(new IOMatch(in31,in3,MatchingType.EXACT));
		
		expected.addIOMatching(new IOMatch(out11,out1,MatchingType.EXACT));
		expected.addIOMatching(new IOMatch(out21,out2,MatchingType.EXACT));
		expected.addIOMatching(new IOMatch(out31,out3,MatchingType.EXACT));
		
		//DebugPrinter.print("expected\n" + expected.toString(), 1);

		// test notNull
		assertNotNull("test not null OK", matching);
		
		assertTrue("test valueOf TypeOfIO - input",new IOMatch(in11,in1,MatchingType.EXACT).getTypeOfIO()==IOMatch.INPUT);
		assertTrue("test valueOf TypeOfIO - output",new IOMatch(out11,out1,MatchingType.EXACT).getTypeOfIO()==IOMatch.OUTPUT);
		
		// testEquals
		assertEquals("test equals", matching, expected);
		
		
	}

	

	/**
	 * Test method for matching with ExactSemanticMatcher (Service Level)
	 * 
	 * @throws SemanticTypeNotKnowException
	 */
	@Test
	public final void testPartialExactMatch()
			throws SemanticTypeNotKnowException {
		// Reference Capa
		Input in1 = new Input("in1",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in2 = new Input("in2",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in3 = new Input("in3",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out1 = new Output("out1",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out2 = new Output("out2",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out3 = new Output("out3",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa1 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa1");

		capa1.addInput(in1);
		capa1.addInput(in2);
		capa1.addInput(in3);

		capa1.addOutput(out1);
		capa1.addOutput(out2);
		capa1.addOutput(out3);

		// Reference Capa 2
		Input in11 = new Input("in11",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in21 = new Input("in21",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in31 = new Input("in31",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out11 = new Output("out11",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out21 = new Output("out21",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out31 = new Output("out31",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa11 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa2", null, "capa11");

		capa11.addInput(in11);
		capa11.addInput(in21);
		capa11.addInput(in31);

		capa11.addOutput(out11);
		capa11.addOutput(out21);
		capa11.addOutput(out31);

		Service s1 = new Service("Tototitoa");
		s1.setType("hye#ho");
		s1.addProvidedCapability(capa1);
		s1.addCapability(capa11);

		// Other Capa
		Input in1a = new Input("in1a",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in2a = new Input("in2a",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in3a = new Input("in3a",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out1a = new Output("out1a",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out2a = new Output("out2a",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out3a = new Output("out3a",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa1a = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa1a");

		capa1a.addInput(in1a);
		capa1a.addInput(in2a);
		capa1a.addInput(in3a);

		capa1a.addOutput(out1a);
		capa1a.addOutput(out2a);
		capa1a.addOutput(out3a);

		// Other Capa
		Input in11a = new Input("in11a",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in21a = new Input("in21a",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in31a = new Input("in31a",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out11a = new Output("out11a",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out21a = new Output("out21a",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out31a = new Output("out31a",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa11a = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa2", null,
				"capa11a");

		capa11a.addInput(in11a);
		capa11a.addInput(in21a);
		capa11a.addInput(in31a);

		capa11a.addOutput(out11a);
		capa11a.addOutput(out21a);
		capa11a.addOutput(out31a);

		UserTask u1 = new UserTask(new Service("Usertask titoa"));
		u1.addRequiredCapability(capa1a);
		u1.addCapability(capa11a);

		ServiceMatch sm = matcher.serviceMatch(s1, u1);

		assertNotNull("test match capa 1", capa1.match(capa1a, matcher));
		assertNotNull("test match capa 2", capa11.match(capa11a, matcher));

		// DebugPrinter.print("sm =\n"+sm,1);

		// test not null
		assertNotNull("test not null", sm);

		CapabilityMatch expected = new CapabilityMatch(capa1a,capa1, MatchingType.EXACT);

		expected.addIOMatching(new IOMatch(in1a,in1,MatchingType.EXACT));
		expected.addIOMatching(new IOMatch(in2a,in2,MatchingType.EXACT));
		expected.addIOMatching(new IOMatch(in3a,in3,MatchingType.EXACT));
		
		expected.addIOMatching(new IOMatch(out1a,out1,MatchingType.EXACT));
		expected.addIOMatching(new IOMatch(out2a,out2,MatchingType.EXACT));
		expected.addIOMatching(new IOMatch(out3a,out3,MatchingType.EXACT));

		
		CapabilityMatch expected2 = new CapabilityMatch(capa11a,capa11, MatchingType.EXACT);		
		
		expected2.addIOMatching(new IOMatch(in11a,in11,MatchingType.EXACT));
		expected2.addIOMatching(new IOMatch(in21a,in21,MatchingType.EXACT));
		expected2.addIOMatching(new IOMatch(in31a,in31,MatchingType.EXACT));
		
		expected2.addIOMatching(new IOMatch(out11a,out11,MatchingType.EXACT));
		expected2.addIOMatching(new IOMatch(out21a,out21,MatchingType.EXACT));
		expected2.addIOMatching(new IOMatch(out31a,out31,MatchingType.EXACT));


		ServiceMatch expected_s = new ServiceMatch(u1, s1, MatchingType.EXACT);
		expected_s.addCapabilityMatch(expected);
		expected_s.addCapabilityMatch(expected2);

		// test equality
		assertEquals("CapabilityMatch 1", sm.getCapabilityMatches().get(0),
				expected);
		assertEquals("CapabilityMatch 2", sm.getCapabilityMatches().get(1),
				expected2);
		//DebugPrinter.print("expected service = \n"+expected_s,DebugPrinter.INFO);
		assertEquals("Test equality", sm, expected_s);
	}

	

}
